import inspect
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadObject


class QEventProxy(AbjadObject):
    '''Proxies a `QEvent`, mapping that QEvent's offset with the range of its
    beatspan to the range 0-1:

    ::

        >>> from abjad.tools import quantizationtools
        >>> q_event = quantizationtools.PitchedQEvent(130, [0, 1, 4])
        >>> proxy = quantizationtools.QEventProxy(q_event, 0.5)
        >>> proxy
        quantizationtools.QEventProxy(
            quantizationtools.PitchedQEvent(
                durationtools.Offset(130, 1),
                (NamedChromaticPitch("c'"), NamedChromaticPitch("cs'"), NamedChromaticPitch("e'")),
                attachments=()
                ),
            durationtools.Offset(1, 2)
            )

    Not composer-safe.

    Used internally by ``Quantizer``.

    Returns `QEventProxy` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_offset', '_q_event')

    ### INITIALIZER ###

    def __init__(self, *args):
        from abjad.tools import quantizationtools

        if len(args) == 2:
            q_event, offset = args[0], durationtools.Offset(args[1])
            assert isinstance(q_event, quantizationtools.QEvent)
            assert 0 <= offset <= 1
        elif len(args) == 3:
            q_event, minimum, maximum = args[0], \
                durationtools.Offset(args[1]), \
                durationtools.Offset(args[2])
            assert isinstance(q_event, quantizationtools.QEvent)
            assert minimum <= q_event.offset <= maximum
            offset = (q_event.offset - minimum) / (maximum - minimum)
        else:
            raise ValueError

        self._q_event = q_event
        self._offset = offset

    ### SPECIAL METHODS ###

    def __eq__(self, other):
        if type(self) == type(other):
            if self.offset == other.offset:
                if self.q_event == other.q_event:
                    return True
        return False

    def __getnewargs__(self):
        return (self.q_event, self.offset)

    def __getstate__(self):
        state = {}
        for klass in inspect.getmro(self.__class__):
            if hasattr(klass, '__slots__'):
                for slot in klass.__slots__:
                    if slot not in state:
                        state[slot] = getattr(self, slot)
        return state

    def __repr__(self):
        return '\n'.join(self._get_tools_package_qualified_repr_pieces())

    def __setstate__(self, state):
        for key, value in state.iteritems():
            setattr(self, key, value)

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _positional_argument_names(self):
        return ('q_event', 'offset')

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def index(self):
        return self._q_event.index

    @property
    def offset(self):
        return self._offset

    @property
    def q_event(self):
        return self._q_event



from abjad.tools import componenttools


def divide_leaves_in_expr_meiotically(expr, n=2):
    r'''.. versionadded:: 1.1

    Divide leaves meiotically in `expr` `n` times::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> beamtools.BeamSpanner(staff.leaves)
        BeamSpanner(c'8, d'8, e'8, f'8)
        >>> f(staff)
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
        }

    ::

        >>> leaftools.divide_leaves_in_expr_meiotically(staff[2:], n=4)

    ::

        >>> f(staff)
        \new Staff {
            c'8 [
            d'8
            e'32
            e'32
            e'32
            e'32
            f'32
            f'32
            f'32
            f'32 ]
        }

    Replace every leaf in `expr` with `n` new leaves.

    Preserve parentage and spanners.

    Allow divisions into only ``1, 2, 4, 8, 16, ...`` and other
    nonnegative integer powers of ``2``.

    Produce only leaves and never tuplets or other containers.

    Return none.

    .. versionchanged:: 2.0
        renamed ``leaftools.meiose()`` to
        ``leaftools.divide_leaves_in_expr_meiotically()``.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import leaftools

    # can not wrap with update control because ofextend_in_parent_of_component()
    for leaf in iterationtools.iterate_leaves_in_expr(expr, reverse=True):
        leaftools.divide_leaf_meiotically(leaf, n)



def all_intervals_are_contiguous(intervals):
    '''True when all intervals in `intervals` are contiguous and non-overlapping.
    '''

    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    for i in range(1, len(tree)):
        if tree[i].start != tree[i-1].stop:
            return False

    return True



from abjad.tools import componenttools
from abjad.tools import iterationtools
from abjad.tools import pitchtools
from abjad.tools import sequencetools


def are_stepwise_descending_notes(*expr):
    '''.. versionadded:: 2.0

    True when notes in `expr` are stepwise descending::

        >>> notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
        >>> t = Staff(list(reversed(notes)))
        >>> tonalitytools.are_stepwise_descending_notes(t[:])
        True

    Otherwise false::

        >>> tonalitytools.are_stepwise_descending_notes(Note("c'4"), Note("c'4"))
        False

    .. versionchanged:: 2.0
        renamed ``tonalitytools.are_stepwise_descending()`` to
        ``tonalitytools.are_stepwise_descending_notes()``.
    '''

    for left, right in sequencetools.iterate_sequence_pairwise_strict(
        iterationtools.iterate_notes_in_expr(expr)):
        try:
            assert not (left.written_pitch == right.written_pitch)
            mdi = pitchtools.calculate_melodic_diatonic_interval(left, right)
            assert mdi.number == -2
        except AssertionError:
            return False

    return True



def is_pitch_carrier(expr):
    '''.. versionadded:: 1.1

    True when `expr` is an Abjad pitch, note, note-head of chord instance.
    Otherwise false::

        >>> note = Note("c'4")
        >>> pitchtools.is_pitch_carrier(note)
        True

    Return boolean.

    .. versionchanged:: 2.0
        renamed ``pitchtools.is_carrier()`` to
        ``pitchtools.is_pitch_carrier()``.
    '''
    from abjad.tools import chordtools
    from abjad.tools import notetools
    from abjad.tools import pitchtools

    return isinstance(
        expr, (
            pitchtools.NamedChromaticPitch, 
            notetools.Note, 
            notetools.NoteHead, 
            chordtools.Chord
            )
        )



from abjad.tools import componenttools
from abjad.tools import iterationtools


def _withdraw_components_in_expr_from_attached_spanners(components):
    '''Find every spanner contained in 'components'.
    Withdraw all components in 'components' from spanners.
    Return 'components'.
    The operation may leave discontiguous spanners.
    '''
    from abjad.tools.spannertools._withdraw_component_from_attached_spanners \
        import _withdraw_component_from_attached_spanners

    # check components
    assert componenttools.all_are_thread_contiguous_components(components)

    # withdraw from contained spanners
    for component in iterationtools.iterate_components_in_expr(
        components, componenttools.Component):
        _withdraw_component_from_attached_spanners(component)

    # return components
    return components



def get_lilypond_command_mark_format_contributions_for_slot(component, slot):
    '''.. versionadded:: 2.0

    Get LilyPond command mark format contributions for `component` at `slot`.

    Return list.
    '''
    from abjad.tools import marktools

    result = []
    comment_marks = marktools.get_lilypond_command_marks_attached_to_component(component)
    for comment_mark in comment_marks:
        if comment_mark._format_slot == slot:
            result.append(comment_mark.lilypond_format)
    return ['lilypond command marks', result]



from abjad.tools.selectiontools.Selection import Selection


class Parentage(Selection):
    r'''Abjad model of component parentage:

    ::

        >>> score = Score()
        >>> score.append(Staff(r"""\new Voice = "Treble Voice" { c'4 }""",
        ...     name='Treble Staff'))
        >>> score.append(Staff(r"""\new Voice = "Bass Voice" { b,4 }""",
        ...     name='Bass Staff'))
    
    ::

        >>> f(score)
        \new Score <<
            \context Staff = "Treble Staff" {
                \context Voice = "Treble Voice" {
                    c'4
                }
            }
            \context Staff = "Bass Staff" {
                \context Voice = "Bass Voice" {
                    b,4
                }
            }
        >>

    ::

        >>> for x in componenttools.Parentage(score): x
        ... 
        Score<<2>>
        
    ::

        >>> for x in componenttools.Parentage(score['Bass Voice'][0]): x
        ...
        Note('b,4')
        Voice-"Bass Voice"{1}
        Staff-"Bass Staff"{1}
        Score<<2>>

    Parentage is treated as a selection of the component's improper parentage.

    Return parentage instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_component', '_root')

    ### INITIALIZER ###

    def __init__(self, component):
        from abjad.tools import componenttools 
        assert isinstance(component, componenttools.Component)
        music = componenttools.get_improper_parentage_of_component(component)
        Selection.__init__(self, music) 
        self._component = component

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def component(self):
        '''The component from which the selection was derived.
        '''
        return self._component

    @property
    def containment_signature(self):
        r'''.. versionadded:: 1.1

        Containment signature of component::

            >>> score = Score(
            ... r"""\context Staff = "CustomStaff" { """
            ...     r"""\context Voice = "CustomVoice" { c' d' e' f' } }""")
            >>> score.name = 'CustomScore'


        ::

            >>> f(score)
            \context Score = "CustomScore" <<
                \context Staff = "CustomStaff" {
                    \context Voice = "CustomVoice" {
                        c'4
                        d'4
                        e'4
                        f'4
                    }
                }
            >>

        ::

            >>> score.leaves[0].parentage.containment_signature
            ContainmentSignature(Note-..., Voice-'CustomVoice', Staff-..., Score-'CustomScore')

        Return containment signature object.
        '''
        from abjad.tools import componenttools
        from abjad.tools import scoretools
        from abjad.tools import stafftools
        from abjad.tools import voicetools
        signature = componenttools.ContainmentSignature()
        signature._self = self[0]._id_string
        for component in self:
            if isinstance(component, voicetools.Voice) and signature._voice is None:
                signature._voice = component._id_string
            elif isinstance(component, stafftools.Staff) and signature._staff is None:
                # leaves inside different staves have diff containment signatures regardless of staff name
                signature._staff = '{}-{}'.format(component._class_name, id(component))
            elif isinstance(component, scoretools.StaffGroup) and signature._staff_group is None:
                signature._staff_group = component._id_string
            elif isinstance(component, scoretools.Score) and signature._score is None:
                signature._score = component._id_string
        else:
            # root components must be the same object for containment signatures to compare true
            signature._root = id(component)
            signature._root_str = component._id_string
        return signature

    @property
    def depth(self):
        '''Length of proper parentage of component.

        Return nonnegative integer.
        '''
        return len(self[1:])

    @property
    def is_orphan(self):
        '''True when component has no parent.
        Otherwise false.

        Return boolean.
        '''
        return self.parent is None

    @property
    def parent(self):
        '''Parent of component or none when component is orphan.
        
        Return component or none.
        '''
        if 1 < len(self):
            return self[1]

    @property
    def parentage_signature(self):
        r'''.. versionadded:: 1.1

        Parentage signature of component::

            >>> tuplet = tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
            >>> staff = Staff([tuplet])
            >>> note = staff.leaves[0]
            >>> print note.parentage.parentage_signature
                staff: Staff-...
                self: Note-...

        Return parentage signature.
        '''
        from abjad.tools import componenttools
        from abjad.tools import scoretools
        from abjad.tools import stafftools
        from abjad.tools import voicetools
        signature = componenttools.ContainmentSignature()
        signature._self = self[0]._id_string
        for component in self:
            if isinstance(component, voicetools.Voice) and not signature._voice:
                signature._voice = component._id_string
            elif isinstance(component, stafftools.Staff) and not signature._staff:
                signature._staff = component._id_string
            elif isinstance(component, scoretools.StaffGroup) and not signature._staff_group:
                signature._staff_group = component._id_string
            elif isinstance(component, scoretools.Score) and not signature._score:
                signature._score = component._id_string
        else:
            # root components must be manifestly equal to compare true
            signature._root = id(component)
            signature._root_str = component._id_string
        return signature

    @property
    def root(self):
        '''Last element in parentage.
        '''
        return self[-1]

    @property
    def score_index(self):
        r'''Score index of component::

            >>> staff_1 = Staff(r"\times 2/3 { c'8 d'8 e'8 } \times 2/3 { f'8 g'8 a'8 }")
            >>> staff_2 = Staff(r"\times 2/3 { b'8 c''8 d''8 }")
            >>> score = Score([staff_1, staff_2])

        ::

            >>> f(score)
            \new Score <<
                \new Staff {
                    \times 2/3 {
                        c'8
                        d'8
                        e'8
                    }
                    \times 2/3 {
                        f'8
                        g'8
                        a'8
                    }
                }
                \new Staff {
                    \times 2/3 {
                        b'8
                        c''8
                        d''8
                    }
                }
            >>

        ::

            >>> for leaf in score.leaves:
            ...     leaf, leaf.parentage.score_index
            ...
            (Note("c'8"), (0, 0, 0))
            (Note("d'8"), (0, 0, 1))
            (Note("e'8"), (0, 0, 2))
            (Note("f'8"), (0, 1, 0))
            (Note("g'8"), (0, 1, 1))
            (Note("a'8"), (0, 1, 2))
            (Note("b'8"), (1, 0, 0))
            (Note("c''8"), (1, 0, 1))
            (Note("d''8"), (1, 0, 2))

        Return tuple of zero or more nonnegative integers.
        '''
        result = []
        cur = self[0]
        for parent in self[1:]:
            index = parent.index(cur)
            result.insert(0, index)
            cur = parent
        result = tuple(result)
        return result

    @property
    def tuplet_depth(self):
        '''Tuplet-depth of component::

            >>> tuplet = tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
            >>> staff = Staff([tuplet])
            >>> note = staff.leaves[0]

        ::

            >>> note.parentage.tuplet_depth
            1

        ::

            >>> tuplet.parentage.tuplet_depth
            0

        ::

            >>> staff.parentage.tuplet_depth
            0

        Return nonnegative integer.
        '''
        from abjad.tools import tuplettools
        from abjad.tools import componenttools
        result = 0
        # should probably interate up to only first parallel container in parentage.
        # note that we probably need a named idea for 'parentage up to first parallel container'.
        for parent in self[1:]:
            if isinstance(parent, tuplettools.Tuplet):
                result += 1
        return result



def get_improper_descendents_of_component_that_stop_with_component(component):
    r'''.. versionadded:: 2.9

    Get improper descendents of `component` that stop with `component`::

        >>> staff = Staff(r"c' << \new Voice { d' } \new Voice { e' } >> f'")

    ::

        >>> f(staff)
        \new Staff {
            c'4
            <<
                \new Voice {
                    d'4
                }
                \new Voice {
                    e'4
                }
            >>
            f'4
        }

    ::

        >>> componenttools.get_improper_descendents_of_component_that_stop_with_component(staff)
        [Staff{3}, Note("f'4")]

    Return list of `component` together with proper contents that stop with `component`.

    .. versionchanged:: 2.9
        renamed ``componenttools.get_improper_contents_of_component_that_stop_with_component()`` to
        ``componenttools.get_improper_descendents_of_component_that_stop_with_component()``.
    '''
    from abjad.tools import containertools

    # initialize result
    result = []

    # add component
    result.append(component)

    # add proper contents that stop with component
    if isinstance(component, containertools.Container):
        if component.is_parallel:
            duration = component.preprolated_duration
            for x in component:
                if x.preprolated_duration == duration:
                    result.extend(get_improper_descendents_of_component_that_stop_with_component(x))
        elif component:
            result.extend(get_improper_descendents_of_component_that_stop_with_component(component[-1]))

    # return result
    return result



def get_stem_tremolo_format_contributions(component):
    '''.. versionadded:: 2.0

    Get stem tremolo format contributions for `component`.
       
    Return list.
    '''
    from abjad.tools import marktools

    result = []

    if marktools.is_component_with_stem_tremolo_attached(component):
        stem_tremolo = marktools.get_stem_tremolo_attached_to_component(component)
        result.append(stem_tremolo.lilypond_format)

    return ['stem tremolo', result]



def explode_intervals_compactly(intervals):
    '''Explode the intervals in `intervals` into n non-overlapping trees,
    where n is the maximum depth of `intervals`.

    The algorithm will attempt to insert the exploded intervals
    into the lowest-indexed resultant tree with free space.

    Return an array of `TimeIntervalTree` instances.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    depth_tree = timeintervaltools.compute_depth_of_intervals(tree)
    max_depth = max([x['depth'] for x in depth_tree])

    layers = [[] for _ in range(max_depth)]
    
    for interval in tree:
        for layer in layers:
            if not len(layer):
                layer.append(interval)
                break
            elif not layer[-1].is_overlapped_by_interval(interval):
                layer.append(interval)
                break

    return [timeintervaltools.TimeIntervalTree(layer) for layer in layers]



from abjad.tools import componenttools


def iterate_thread_from_component(component, klass=None, reverse=False):
    r'''.. versionadded:: 2.10

    Itearte thread forward from `component` and yield instances of `klass`::

        >>> container = Container(Voice(notetools.make_repeated_notes(2)) * 2)
        >>> container.is_parallel = True
        >>> container[0].name = 'voice 1'
        >>> container[1].name = 'voice 2'
        >>> staff = Staff(container * 2)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)

    ::

        >>> f(staff)
        \new Staff {
            <<
                \context Voice = "voice 1" {
                    c'8
                    d'8
                }
                \context Voice = "voice 2" {
                    e'8
                    f'8
                }
            >>
            <<
                \context Voice = "voice 1" {
                    g'8
                    a'8
                }
                \context Voice = "voice 2" {
                    b'8
                    c''8
                }
            >>
        }

    Starting from the first leaf in score. ::

        >>> for x in iterationtools.iterate_thread_from_component(
        ...     staff.leaves[0], Note):
        ...     x
        ...
        Note("c'8")
        Note("d'8")
        Note("g'8")
        Note("a'8")

    Starting from the second leaf in score. ::

        >>> for x in iterationtools.iterate_thread_from_component(
        ...     staff.leaves[1], Note):
        ...     x
        ...
        Note("d'8")
        Note("g'8")
        Note("a'8")

    Yield all components in thread. ::

        >>> for x in iterationtools.iterate_thread_from_component(
        ...     staff.leaves[0]):
        ...     x
        ...
        Note("c'8")
        Voice-"voice 1"{2}
        Note("d'8")
        Voice-"voice 1"{2}
        Note("g'8")
        Note("a'8")

    Iterate thread backward from `component` and yield instances of `klass`,
    starting from the last leaf in score. ::

        >>> for x in iterationtools.iterate_thread_from_component(
        ...     staff.leaves[-1], Note, reverse=True):
        ...     x
        Note("c''8")
        Note("b'8")
        Note("f'8")
        Note("e'8")

    Yield all components in thread::

        >>> for x in iterationtools.iterate_thread_from_component(
        ...     staff.leaves[-1], reverse=True):
        ...     x
        Note("c''8")
        Voice-"voice 2"{2}
        Note("b'8")
        Voice-"voice 2"{2}
        Note("f'8")
        Note("e'8")

    Return generator.
    '''
    from abjad.tools import iterationtools

    # set default class
    if klass is None:
        klass = componenttools.Component

    # save thread signature of input component
    component_thread_signature = component.parentage.containment_signature

    # iterate component depth-first allowing to crawl UP into score
    if not reverse:
        for x in iterationtools.iterate_components_depth_first(component, capped=False):
            if isinstance(x, klass):
                if x.parentage.containment_signature == component_thread_signature:
                    yield x
    else:
        for x in iterationtools.iterate_components_depth_first(component, capped=False, direction=Right):
            if isinstance(x, klass):
                if x.parentage.containment_signature == component_thread_signature:
                    yield x



from abjad.tools import durationtools


def concatenate_trees(trees, padding=0):
    '''Merge all trees in `trees`, offsetting each subsequent tree
    to start after the previous.

    Return TimeIntervalTree.
    '''
    from abjad.tools import timeintervaltools

    assert all([isinstance(tree, timeintervaltools.TimeIntervalTree) for tree in trees])
    padding = durationtools.Duration(padding)
    assert 0 <= padding

    output_tree = timeintervaltools.TimeIntervalTree(
        timeintervaltools.shift_aggregate_offset_to_rational(trees[0], 0))
    for tree in trees[1:]:
        output_tree = timeintervaltools.TimeIntervalTree([
            output_tree,
            timeintervaltools.shift_aggregate_offset_to_rational(
                tree, output_tree.latest_stop + padding)
        ])

    return output_tree



from abjad.tools.spannertools.Spanner import Spanner


class HiddenStaffSpanner(Spanner):
    r'''Abjad hidden staff spanner::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> spannertools.HiddenStaffSpanner(staff[:2])
        HiddenStaffSpanner(c'8, d'8)

    ::

        >>> f(staff)
        \new Staff {
            \stopStaff
            c'8
            d'8
            \startStaff
            e'8
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Hide staff behind leaves in spanner.

    Return hidden staff spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None):
        Spanner.__init__(self, components)

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        pass

    def _format_after_leaf(self, leaf):
        result = []
        if self._is_my_last_leaf(leaf):
            result.append(r'\startStaff')
        return result

    def _format_before_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\stopStaff')
        return result



from abjad.tools.spannertools.Spanner import Spanner


class OctavationSpanner(Spanner):
    r'''Abjad octavation spanner::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> spanner = spannertools.OctavationSpanner(staff[:], start=1)

    ::

        >>> f(staff)
        \new Staff {
            \ottava #1
            c'8
            d'8
            e'8
            f'8
            \ottava #0
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return octavation spanner.
    '''

    ### INITIALIZER ###

    # TODO: Set start to 1 (and stop to 0) by default. #
    def __init__(self, components=None, start=0, stop=0):
        Spanner.__init__(self, components)
        self.start = start
        self.stop = stop

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new.start = self.start
        new.stop = self.stop

    def _format_after_leaf(self, leaf):
        result = []
        result.extend(Spanner._format_after_leaf(self, leaf))
        if self._is_my_last_leaf(leaf):
            result.append(r'\ottava #%s' % self.stop)
        return result

    def _format_before_leaf(self, leaf):
        result = []
        result.extend(Spanner._format_before_leaf(self, leaf))
        if self._is_my_first_leaf(leaf):
            result.append(r'\ottava #%s' % self.start)
        return result

    ### READ / WRITE PUBLIC PROPERTIES ###

    @apply
    def start():
        def fget(self):
            r'''Get octavation start::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> octavation = spannertools.OctavationSpanner(staff[:], start=1)
                >>> octavation.start
                1

            Set octavation start::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> octavation = spannertools.OctavationSpanner(staff[:], start=1)
                >>> octavation.start
                1

            Set integer.
            '''
            return self._start
        def fset(self, arg):
            assert isinstance(arg, (int, type(None)))
            self._start = arg
        return property(**locals())

    @apply
    def stop():
        def fget(self):
            r'''Get octavation stop::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> octavation = spannertools.OctavationSpanner(staff[:], start=2, stop=1)
                >>> octavation.stop
                1

            Set octavation stop::

                >>> staff = Staff("c'8 d'8 e'8 f'8")
                >>> octavation = spannertools.OctavationSpanner(staff[:], start=2, stop=1)
                >>> octavation.stop = 0
                >>> octavation.stop
                0

            Set integer.
            '''
            return self._stop
        def fset(self, arg):
            assert isinstance(arg, (int, type(None)))
            self._stop = arg
        return property(**locals())



from abjad.tools import resttools


def iterate_rests_in_expr(expr, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 2.10

    Iterate rests forward in `expr`::

        >>> staff = Staff("<e' g' c''>8 a'8 r8 <d' f' b'>8 r2")

    ::

        >>> f(staff)
        \new Staff {
            <e' g' c''>8
            a'8
            r8
            <d' f' b'>8
            r2
        }

    ::

        >>> for rest in iterationtools.iterate_rests_in_expr(staff):
        ...   rest
        Rest('r8')
        Rest('r2')

    Iterate rests backward in `expr`::

        >>> for rest in iterationtools.iterate_rests_in_expr(staff, reverse=True):
        ...   rest
        Rest('r2')
        Rest('r8')

    Ignore threads.

    Return generator.
    '''
    from abjad.tools import iterationtools

    return iterationtools.iterate_components_in_expr(
        expr, klass=resttools.Rest, reverse=reverse, start=start, stop=stop)



from abjad import *
import py.test


def test_componenttools_is_immediate_temporal_successor_of_component_01( ):
    '''The second of two leaves in the same voice is
    the immediate temporal follower of the first.
    '''

    t = Voice([Note(i, (1, 8)) for i in range(4)])

    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[1], t[2])
    assert componenttools.is_immediate_temporal_successor_of_component(t[2], t[3])

    r'''
    \new Voice {
        c'8
        cs'8
        d'8
        ef'8
    }
    '''


def test_componenttools_is_immediate_temporal_successor_of_component_02( ):
    '''The second of two leaves in the same staff is
    the immediate temporal follower of the first.
    '''

    t = Staff([Note(i, (1, 8)) for i in range(4)])

    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[1], t[2])
    assert componenttools.is_immediate_temporal_successor_of_component(t[2], t[3])

    r'''
    \new Staff {
        c'8
        cs'8
        d'8
        ef'8
    }
    '''


def test_componenttools_is_immediate_temporal_successor_of_component_03( ):
    '''The second of two leaves in the same sequential is
    the immediate temporal follower of the first.
    '''

    t = Staff([Note(i, (1, 8)) for i in range(4)])

    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[1], t[2])
    assert componenttools.is_immediate_temporal_successor_of_component(t[2], t[3])

    r'''
    {
        c'8
        cs'8
        d'8
        ef'8
    }
    '''


def test_componenttools_is_immediate_temporal_successor_of_component_04( ):
    '''The second of two leaves in the same tuplet is
    the immediate temporal follower of the first.
    '''

    t = tuplettools.FixedDurationTuplet(Duration(2, 8), [Note(i, (1, 8)) for i in range(3)])

    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[1], t[2])

    r'''
    \times 2/3 {
        c'8
        cs'8
        d'8
    }
    '''


def test_componenttools_is_immediate_temporal_successor_of_component_05( ):
    '''The second sequential and the first note of the second sequential
    both temporally follow the first sequential and the last
    note of the first sequential immediately.
    '''

    s1 = Container([Note(i, (1, 8)) for i in range(4)])
    s2 = Container([Note(i, (1, 8)) for i in range(4, 8)])
    t = Voice([s1, s2])

    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0][-1], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0][-1], t[1][0])

    r'''
    \new Voice {
        {
            c'8
            cs'8
            d'8
            ef'8
        }
        {
            e'8
            f'8
            fs'8
            g'8
        }
    }
    '''


def test_componenttools_is_immediate_temporal_successor_of_component_06( ):
    '''The second tuplet and the first note of the second tuplet
    both temporally follow the first tuplet and the last
    note of the first tuplet immediately.
    '''

    t1 = tuplettools.FixedDurationTuplet(Duration(2, 8), [Note(i, (1, 8)) for i in range(3)])
    t2 = tuplettools.FixedDurationTuplet(Duration(2, 8), [Note(i, (1, 8)) for i in range(3, 6)])
    t = Voice([t1, t2])

    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0][-1], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0][-1], t[1][0])

    r'''
    \new Voice {
        \times 2/3 {
            c'8
            cs'8
            d'8
        }
        \times 2/3 {
            ef'8
            e'8
            f'8
        }
    }
    '''


def test_componenttools_is_immediate_temporal_successor_of_component_07( ):
    '''The second (anonymous) voice and the first note of the
    second (anonymous) voice both temporally follow the
    first (anonymous) voice and the last note of the
    first (anonymous) voice immediately.
    '''

    v1 = Voice([Note(i, (1, 8)) for i in range(4)])
    v2 = Voice([Note(i, (1, 8)) for i in range(4, 8)])
    t = Staff([v1, v2])

    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0][-1], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0][-1], t[1][0])

    r'''
    \new Staff {
        \new Voice {
            c'8
            cs'8
            d'8
            ef'8
        }
        \new Voice {
            e'8
            f'8
            fs'8
            g'8
        }
    }
    '''


def test_componenttools_is_immediate_temporal_successor_of_component_08( ):
    '''The second (like-named) voice and the first note of the
    second (like-named) voice both temporally follow the
    first (like-named) voice and the last note of the
    first (like-named) voice immediately.
    '''

    v1 = Voice([Note(i, (1, 8)) for i in range(4)])
    v1.name = 'foo'
    v2 = Voice([Note(i, (1, 8)) for i in range(4, 88)])
    v2.name = 'foo'
    t = Staff([v1, v2])

    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0][-1], t[1])

    r'''
    \new Staff {
        \context Voice = "foo" {
            c'8
            cs'8
            d'8
            ef'8
        }
        \context Voice = "foo" {
            e'8
            f'8
            fs'8
            g'8
        }
    }
    '''


def test_componenttools_is_immediate_temporal_successor_of_component_09( ):
    '''The second (differently named) voice and the first note of the
    second (differently named) voice both temporally follow the
    first (differently named) voice and the last note of the
    first (differently named) voice immediately.
    '''

    v1 = Voice([Note(i, (1, 8)) for i in range(4)])
    v1.name = 'foo'
    v2 = Voice([Note(i, (1, 8)) for i in range(4, 88)])
    v2.name = 'bar'
    t = Staff([v1, v2])

    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0][-1], t[1])

    r'''
    \new Staff {
        \context Voice = "foo" {
            c'8
            cs'8
            d'8
            ef'8
        }
        \context Voice = "bar" {
            e'8
            f'8
            fs'8
            g'8
        }
    }
    '''


def test_componenttools_is_immediate_temporal_successor_of_component_10( ):
    '''Each of ...
        * the first (anonymous) staff
        * the first (anonymous) voice
        * the last note in the first (anonymous) voice
    ... is followed in immediate temporal order by each of ...
        * the second (anonymous) staff
        * the second (anonymous) voice
        * the first note in the second (anonymous) voice.
    '''

    v1 = Voice([Note(i, (1, 8)) for i in range(4)])
    v2 = Voice([Note(i, (1, 8)) for i in range(4, 8)])
    s1 = Staff([v1])
    s2 = Staff([v2])
    seq = Container([s1, s2])

    assert componenttools.is_immediate_temporal_successor_of_component(seq[0], seq[1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0], seq[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0], seq[1][0][0])

    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0], seq[1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0], seq[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0], seq[1][0][0])

    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0][-1], seq[1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0][-1], seq[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0][-1], seq[1][0][0])

    r'''
    {
        \new Staff {
            \new Voice {
                c'8
                cs'8
                d'8
                ef'8
            }
        }
        \new Staff {
            \new Voice {
                e'8
                f'8
                fs'8
                g'8
            }
        }
    }
    '''


def test_componenttools_is_immediate_temporal_successor_of_component_11( ):
    '''Everything at the beginning of the second staff temporally
    follows everything at the end of the first staff immediately.
    '''

    vl1 = Voice([Note(i, (1, 8)) for i in range(4)])
    vl2 = Voice([Note(i, (1, 8)) for i in range(4, 8)])
    vh1 = Voice([Note(i, (1, 8)) for i in range(12, 16)])
    vh2 = Voice([Note(i, (1, 8)) for i in range(16, 20)])
    s1 = Staff([vh1, vl1])
    s1.is_parallel = True
    s2 = Staff([vl2, vh2])
    s2.is_parallel = True
    seq = Container([s1, s2])

    assert componenttools.is_immediate_temporal_successor_of_component(seq[0], seq[1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0], seq[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0], seq[1][0][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0], seq[1][1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0], seq[1][1][0])

    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0], seq[1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0], seq[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0], seq[1][0][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0], seq[1][1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0], seq[1][1][0])

    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0][-1], seq[1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0][-1], seq[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0][-1], seq[1][0][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0][-1], seq[1][1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][0][-1], seq[1][1][0])

    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][1], seq[1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][1], seq[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][1], seq[1][0][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][1], seq[1][1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][1], seq[1][1][0])

    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][1][-1], seq[1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][1][-1], seq[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][1][-1], seq[1][0][0])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][1][-1], seq[1][1])
    assert componenttools.is_immediate_temporal_successor_of_component(seq[0][1][-1], seq[1][1][0])

    r'''
    {
        \new Staff <<
            \new Voice {
                c''8
                cs''8
                d''8
                ef''8
            }
            \new Voice {
                c'8
                cs'8
                d'8
                ef'8
            }
        >>
        \new Staff <<
            \new Voice {
                e'8
                f'8
                fs'8
                g'8
            }
            \new Voice {
                e''8
                f''8
                fs''8
                g''8
            }
        >>
    }
    '''


def test_componenttools_is_immediate_temporal_successor_of_component_12( ):
    '''Everything at the beginning of the second sequential temporally
    follows everything at the end of the first sequential immediately.
    '''

    s1 = Container([Note(i, (1, 8)) for i in range(4)])
    s1 = Container([s1])
    s2 = Container([Note(i, (1, 8)) for i in range(4, 8)])
    s2 = Container([s2])
    t = Voice([s1, s2])

    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0], t[1][0][0])

    assert componenttools.is_immediate_temporal_successor_of_component(t[0][0], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0][0], t[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0][0], t[1][0][0])

    assert componenttools.is_immediate_temporal_successor_of_component(t[0][0][-1], t[1])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0][0][-1], t[1][0])
    assert componenttools.is_immediate_temporal_successor_of_component(t[0][0][-1], t[1][0][0])

    r'''
    \new Voice {
        {
            {
                c'8
                cs'8
                d'8
                ef'8
            }
        }
        {
            {
                e'8
                f'8
                fs'8
                g'8
            }
        }
    }
    '''



import py.test
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_LilyPondParser__spanners__TieSpanner_01():
    target = Container([Note(0, 1), Note(0, 1)])
    tietools.TieSpanner(target[:])
    parser = LilyPondParser()
    result = parser(target.lilypond_format)
    assert target.lilypond_format == result.lilypond_format and target is not result


def test_LilyPondParser__spanners__TieSpanner_02():
    input = r'{ c ~ }'
    assert py.test.raises(Exception, 'LilyPondParser()(input)')


def test_LilyPondParser__spanners__TieSpanner_03():
    input = r'{ ~ c }'
    assert py.test.raises(Exception, 'LilyPondParser()(input)')


def test_LilyPondParser__spanners__TieSpanner_04():
    '''With direction.'''
    target = Container([Note(0, 1), Note(0, 1)])
    tietools.TieSpanner(target[:], direction=Up)
    parser = LilyPondParser()
    result = parser(target.lilypond_format)
    assert target.lilypond_format == result.lilypond_format and target is not result


def test_LilyPondParser__spanners__TieSpanner_05():
    '''With direction.'''
    target = Container([Note(0, 1), Note(0, 1)])
    tietools.TieSpanner(target[:], direction=Down)
    parser = LilyPondParser()
    result = parser(target.lilypond_format)
    assert target.lilypond_format == result.lilypond_format and target is not result



import copy
from abjad.tools import leaftools
from abjad.tools.abctools.SortableAttributeEqualityAbjadObject import SortableAttributeEqualityAbjadObject
from abjad.tools.lilypondproxytools import LilyPondTweakReservoir


class NoteHead(SortableAttributeEqualityAbjadObject):
    r'''Abjad model of a note head:

    ::

        >>> notetools.NoteHead(13)
        NoteHead("cs''")

    Note heads are immutable.
    '''

    __slots__ = ('_client', '_is_cautionary', '_is_forced', '_tweak', '_written_pitch')

    def __init__(self, written_pitch=None, client=None,
        is_cautionary=False, is_forced=False, tweak_pairs=()):
        assert isinstance(client, (type(None), leaftools.Leaf))
        self._client = client
        self.written_pitch = written_pitch
        self.is_cautionary = is_cautionary
        self.is_forced = is_forced
        # must assign comparison attribute after written pitch initialization #
        self._comparison_attribute = self.written_pitch
        for tweak_pair in tweak_pairs:
            key, value = tweak_pair
            setattr(self.tweak, key, copy.copy(value))

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        return type(self)(**self.__getnewargs__())

    __deepcopy__ = __copy__

    def __getnewargs__(self):
        args = {
            'written_pitch': self.written_pitch,
            'is_cautionary': self.is_cautionary,
            'is_forced': self.is_forced,
            'tweak_pairs': self.tweak._get_attribute_pairs(),
        }
        return args

    def __repr__(self):
        args = [repr(self._format_string)]
        args.extend(self.tweak._get_attribute_pairs())
        args = ', '.join([str(x) for x in args])
        return '%s(%s)' % (type(self).__name__, args)

    def __str__(self):
        return self._format_string

    ### PRIVATE PROPERTIES ###

    @property
    def _format_string(self):
        result = ''
        if self.written_pitch:
            result = str(self.written_pitch)
            if self.is_forced:
                result += '!'
            if self.is_cautionary:
                result += '?'
        return result

    ### PUBLIC PROPERTIES ###

    @apply
    def is_cautionary():
        def fget(self):
            '''Get cautionary accidental flag::

                >>> note_head = notetools.NoteHead("cs''")
                >>> note_head.is_cautionary
                False

            Set cautionary accidental flag::

                >>> note_head = notetools.NoteHead("cs''")
                >>> note_head.is_cautionary = True

            Return boolean.
            '''
            return self._is_cautionary
        def fset(self, arg):
            self._is_cautionary = bool(arg)
        return property(**locals())

    @apply
    def is_forced():
        '''Get forced accidental flag::

            >>> note_head = notetools.NoteHead("cs''")
            >>> note_head.is_forced
            False

        Set forced accidental flag::

            >>> note_head = notetools.NoteHead("cs''")
            >>> note_head.is_forced = True

        Return boolean.
        '''
        def fget(self):
            return self._is_forced
        def fset(self, arg):
            self._is_forced = bool(arg)
        return property(**locals())

    @property
    def lilypond_format(self):
        '''Read-only LilyPond input format of note head::

            >>> note_head = notetools.NoteHead("cs''")
            >>> note_head.lilypond_format
            "cs''"

        Return string.
        '''
        from abjad.tools.notetools._format_note_head import _format_note_head
        return _format_note_head(self)

    @property
    def named_chromatic_pitch(self):
        '''Read-only named chromatic pitch equal to note head::

            >>> note_head = notetools.NoteHead("cs''")
            >>> note_head.named_chromatic_pitch
            NamedChromaticPitch("cs''")

        Return named chromatic pitch.
        '''
        return self.written_pitch

    @property
    def tweak(self):
        '''Read-only LilyPond tweak reservoir::

            >>> note_head = notetools.NoteHead("cs''")
            >>> note_head.tweak
            LilyPondTweakReservoir()

        Return LilyPond tweak reservoir.
        '''
        if not hasattr(self, '_tweak'):
            self._tweak = LilyPondTweakReservoir()
        return self._tweak

    @apply
    def written_pitch():
        def fget(self):
            '''Get named pitch of note head::

                >>> note_head = notetools.NoteHead("cs''")
                >>> note_head.written_pitch
                NamedChromaticPitch("cs''")

            Set named pitch of note head::

                >>> note_head = notetools.NoteHead("cs''")
                >>> note_head.written_pitch = "d''"
                >>> note_head.written_pitch
                NamedChromaticPitch("d''")

            Set pitch token.
            '''
            return self._written_pitch
        def fset(self, arg):
            from abjad.tools import pitchtools
            written_pitch = pitchtools.NamedChromaticPitch(arg)
            self._written_pitch = written_pitch
        return property(**locals())



# -*- coding: utf-8 -*-
"""
    abjad.docs.ext.autodoc
    ~~~~~~~~~~~~~~~~~~

    Automatically insert docstrings for functions, classes or whole modules into
    the doctree, thus avoiding duplication between docstrings and documentation
    for those who like elaborate docstrings.

    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.

    Modified for Abjad documentation conventions.
"""

import re
import sys
import inspect
import traceback
from types import FunctionType, BuiltinFunctionType, MethodType

from docutils import nodes
from docutils.utils import assemble_option_dict
from docutils.statemachine import ViewList

from sphinx.util import rpartition, force_decode
from sphinx.locale import _
from sphinx.pycode import ModuleAnalyzer, PycodeError
from sphinx.application import ExtensionError
from sphinx.util.nodes import nested_parse_with_titles
from sphinx.util.compat import Directive
#from sphinx.util.inspect import getargspec, isdescriptor, safe_getmembers, \
#     safe_getattr, safe_repr
from sphinx.util.inspect import isdescriptor, safe_getmembers, \
     safe_getattr, safe_repr
from sphinx.util.pycompat import base_exception, class_types
from sphinx.util.docstrings import prepare_docstring

from sphinx.ext.autodoc import Documenter


def getargspec(proc):
    from sphinx.util.inspect import getargspec as sphinx_getargspec
    if proc.__name__ == 'wrapper':
        if proc.__closure__:
            return sphinx_getargspec(proc.__closure__[1].cell_contents)
    return sphinx_getargspec(proc)
    

#: extended signature RE: with explicit module name separated by ::
py_ext_sig_re = re.compile(
    r'''^ ([\w.]+::)?            # explicit module name
          ([\w.]+\.)?            # module and/or class name(s)
          (\w+)  \s*             # thing name
          (?: \((.*)\)           # optional: arguments
           (?:\s* -> \s* (.*))?  #           return annotation
          )? $                   # and nothing more
          ''', re.VERBOSE)


class DefDict(dict):
    """A dict that returns a default on nonexisting keys."""
    def __init__(self, default):
        dict.__init__(self)
        self.default = default
    def __getitem__(self, key):
        try:
            return dict.__getitem__(self, key)
        except KeyError:
            return self.default
    def __nonzero__(self):
        # docutils check "if option_spec"
        return True

identity = lambda x: x


class Options(dict):
    """A dict/attribute hybrid that returns None on nonexisting keys."""
    def __getattr__(self, name):
        try:
            return self[name.replace('_', '-')]
        except KeyError:
            return None


ALL = object()
INSTANCEATTR = object()

def members_option(arg):
    """Used to convert the :members: option to auto directives."""
    if arg is None:
        return ALL
    return [x.strip() for x in arg.split(',')]

def members_set_option(arg):
    """Used to convert the :members: option to auto directives."""
    if arg is None:
        return ALL
    return set(x.strip() for x in arg.split(','))

def bool_option(arg):
    """Used to convert flag options to auto directives.  (Instead of
    directives.flag(), which returns None).
    """
    return True


class AutodocReporter(object):
    """
    A reporter replacement that assigns the correct source name
    and line number to a system message, as recorded in a ViewList.
    """
    def __init__(self, viewlist, reporter):
        self.viewlist = viewlist
        self.reporter = reporter

    def __getattr__(self, name):
        return getattr(self.reporter, name)

    def system_message(self, level, message, *children, **kwargs):
        if 'line' in kwargs and 'source' not in kwargs:
            try:
                source, line = self.viewlist.items[kwargs['line']]
            except IndexError:
                pass
            else:
                kwargs['source'] = source
                kwargs['line'] = line
        return self.reporter.system_message(level, message,
                                            *children, **kwargs)

    def debug(self, *args, **kwargs):
        if self.reporter.debug_flag:
            return self.system_message(0, *args, **kwargs)

    def info(self, *args, **kwargs):
        return self.system_message(1, *args, **kwargs)

    def warning(self, *args, **kwargs):
        return self.system_message(2, *args, **kwargs)

    def error(self, *args, **kwargs):
        return self.system_message(3, *args, **kwargs)

    def severe(self, *args, **kwargs):
        return self.system_message(4, *args, **kwargs)


# Some useful event listener factories for autodoc-process-docstring.

def cut_lines(pre, post=0, what=None):
    """Return a listener that removes the first *pre* and last *post*
    lines of every docstring.  If *what* is a sequence of strings,
    only docstrings of a type in *what* will be processed.

    Use like this (e.g. in the ``setup()`` function of :file:`conf.py`)::

       from sphinx.ext.autodoc import cut_lines
       app.connect('autodoc-process-docstring', cut_lines(4, what=['module']))

    This can (and should) be used in place of :confval:`automodule_skip_lines`.
    """
    def process(app, what_, name, obj, options, lines):
        if what and what_ not in what:
            return
        del lines[:pre]
        if post:
            # remove one trailing blank line.
            if lines and not lines[-1]:
                lines.pop(-1)
            del lines[-post:]
        # make sure there is a blank line at the end
        if lines and lines[-1]:
            lines.append('')
    return process

def between(marker, what=None, keepempty=False, exclude=False):
    """Return a listener that either keeps, or if *exclude* is True excludes,
    lines between lines that match the *marker* regular expression.  If no line
    matches, the resulting docstring would be empty, so no change will be made
    unless *keepempty* is true.

    If *what* is a sequence of strings, only docstrings of a type in *what* will
    be processed.
    """
    marker_re = re.compile(marker)
    def process(app, what_, name, obj, options, lines):
        if what and what_ not in what:
            return
        deleted = 0
        delete = not exclude
        orig_lines = lines[:]
        for i, line in enumerate(orig_lines):
            if delete:
                lines.pop(i - deleted)
                deleted += 1
            if marker_re.match(line):
                delete = not delete
                if delete:
                    lines.pop(i - deleted)
                    deleted += 1
        if not lines and not keepempty:
            lines[:] = orig_lines
        # make sure there is a blank line at the end
        if lines and lines[-1]:
            lines.append('')
    return process


#class Documenter(object):
#    """
#    A Documenter knows how to autodocument a single object type.  When
#    registered with the AutoDirective, it will be used to document objects
#    of that type when needed by autodoc.
#
#    Its *objtype* attribute selects what auto directive it is assigned to
#    (the directive name is 'auto' + objtype), and what directive it generates
#    by default, though that can be overridden by an attribute called
#    *directivetype*.
#
#    A Documenter has an *option_spec* that works like a docutils directive's;
#    in fact, it will be used to parse an auto directive's options that matches
#    the documenter.
#    """
#    #: name by which the directive is called (auto...) and the default
#    #: generated directive name
#    objtype = 'object'
#    #: indentation by which to indent the directive content
#    content_indent = u'   '
#    #: priority if multiple documenters return True from can_document_member
#    priority = 0
#    #: order if autodoc_member_order is set to 'groupwise'
#    member_order = 0
#    #: true if the generated content may contain titles
#    titles_allowed = False
#
#    option_spec = {'noindex': bool_option}
#
#    @staticmethod
#    def get_attr(obj, name, *defargs):
#        """getattr() override for types such as Zope interfaces."""
#        for typ, func in AutoDirective._special_attrgetters.iteritems():
#            if isinstance(obj, typ):
#                return func(obj, name, *defargs)
#        return safe_getattr(obj, name, *defargs)
#
#    @classmethod
#    def can_document_member(cls, member, membername, isattr, parent):
#        """Called to see if a member can be documented by this documenter."""
#        raise NotImplementedError('must be implemented in subclasses')
#
#    def __init__(self, directive, name, indent=u''):
#        self.directive = directive
#        self.env = directive.env
#        self.options = directive.genopt
#        self.name = name
#        self.indent = indent
#        # the module and object path within the module, and the fully
#        # qualified name (all set after resolve_name succeeds)
#        self.modname = None
#        self.module = None
#        self.objpath = None
#        self.fullname = None
#        # extra signature items (arguments and return annotation,
#        # also set after resolve_name succeeds)
#        self.args = None
#        self.retann = None
#        # the object to document (set after import_object succeeds)
#        self.object = None
#        self.object_name = None
#        # the parent/owner of the object to document
#        self.parent = None
#        # the module analyzer to get at attribute docs, or None
#        self.analyzer = None
#
#    def add_line(self, line, source, *lineno):
#        """Append one line of generated reST to the output."""
#        self.directive.result.append(self.indent + line, source, *lineno)
#
#    def resolve_name(self, modname, parents, path, base):
#        """Resolve the module and name of the object to document given by the
#        arguments and the current module/class.
#
#        Must return a pair of the module name and a chain of attributes; for
#        example, it would return ``('zipfile', ['ZipFile', 'open'])`` for the
#        ``zipfile.ZipFile.open`` method.
#        """
#        raise NotImplementedError('must be implemented in subclasses')
#
#    def parse_name(self):
#        """Determine what module to import and what attribute to document.
#
#        Returns True and sets *self.modname*, *self.objpath*, *self.fullname*,
#        *self.args* and *self.retann* if parsing and resolving was successful.
#        """
#        # first, parse the definition -- auto directives for classes and
#        # functions can contain a signature which is then used instead of
#        # an autogenerated one
#        try:
#            explicit_modname, path, base, args, retann = \
#                 py_ext_sig_re.match(self.name).groups()
#        except AttributeError:
#            self.directive.warn('invalid signature for auto%s (%r)' %
#                                (self.objtype, self.name))
#            return False
#
#        # support explicit module and class name separation via ::
#        if explicit_modname is not None:
#            modname = explicit_modname[:-2]
#            parents = path and path.rstrip('.').split('.') or []
#        else:
#            modname = None
#            parents = []
#
#        self.modname, self.objpath = \
#                      self.resolve_name(modname, parents, path, base)
#
#        if not self.modname:
#            return False
#
#        self.args = args
#        self.retann = retann
#        self.fullname = (self.modname or '') + \
#                        (self.objpath and '.' + '.'.join(self.objpath) or '')
#        return True
#
#    def import_object(self):
#        """Import the object given by *self.modname* and *self.objpath* and set
#        it as *self.object*.
#
#        Returns True if successful, False if an error occurred.
#        """
#        try:
#            __import__(self.modname)
#            parent = None
#            obj = self.module = sys.modules[self.modname]
#            for part in self.objpath:
#                parent = obj
#                obj = self.get_attr(obj, part)
#                self.object_name = part
#            self.parent = parent
#            self.object = obj
#            return True
#        # this used to only catch SyntaxError, ImportError and AttributeError,
#        # but importing modules with side effects can raise all kinds of errors
#        except Exception, err:
#            if self.env.app and not self.env.app.quiet:
#                self.env.app.info(traceback.format_exc().rstrip())
#            self.directive.warn(
#                'autodoc can\'t import/find %s %r, it reported error: '
#                '"%s", please check your spelling and sys.path' %
#                (self.objtype, str(self.fullname), err))
#            self.env.note_reread()
#            return False
#
#    def get_real_modname(self):
#        """Get the real module name of an object to document.
#
#        It can differ from the name of the module through which the object was
#        imported.
#        """
#        return self.get_attr(self.object, '__module__', None) or self.modname
#
#    def check_module(self):
#        """Check if *self.object* is really defined in the module given by
#        *self.modname*.
#        """
#        modname = self.get_attr(self.object, '__module__', None)
#        if modname and modname != self.modname:
#            return False
#        return True
#
#    def format_args(self):
#        """Format the argument signature of *self.object*.
#
#        Should return None if the object does not have a signature.
#        """
#        return None
#
#    def format_name(self):
#        """Format the name of *self.object*.
#
#        This normally should be something that can be parsed by the generated
#        directive, but doesn't need to be (Sphinx will display it unparsed
#        then).
#        """
#        # normally the name doesn't contain the module (except for module
#        # directives of course)
#        return '.'.join(self.objpath) or self.modname
#
#    def format_signature(self):
#        """Format the signature (arguments and return annotation) of the object.
#
#        Let the user process it via the ``autodoc-process-signature`` event.
#        """
#        if self.args is not None:
#            # signature given explicitly
#            args = "(%s)" % self.args
#        else:
#            # try to introspect the signature
#            try:
#                args = self.format_args()
#            except Exception, err:
#                self.directive.warn('error while formatting arguments for '
#                                    '%s: %s' % (self.fullname, err))
#                args = None
#
#        retann = self.retann
#
#        result = self.env.app.emit_firstresult(
#            'autodoc-process-signature', self.objtype, self.fullname,
#            self.object, self.options, args, retann)
#        if result:
#            args, retann = result
#
#        if args is not None:
#            return args + (retann and (' -> %s' % retann) or '')
#        else:
#            return ''
#
#    def add_directive_header(self, sig):
#        """Add the directive header and options to the generated content."""
#        domain = getattr(self, 'domain', 'py')
#        directive = getattr(self, 'directivetype', self.objtype)
#        name = self.format_name()
#        self.add_line(u'.. %s:%s:: %s%s' % (domain, directive, name, sig),
#                      '<autodoc>')
#        if self.options.noindex:
#            self.add_line(u'   :noindex:', '<autodoc>')
#        if self.objpath:
#            # Be explicit about the module, this is necessary since .. class::
#            # etc. don't support a prepended module name
#            self.add_line(u'   :module: %s' % self.modname, '<autodoc>')
#
#    def get_doc(self, encoding=None, ignore=1):
#        """Decode and return lines of the docstring(s) for the object."""
#        docstring = self.get_attr(self.object, '__doc__', None)
#        # make sure we have Unicode docstrings, then sanitize and split
#        # into lines
#        if isinstance(docstring, unicode):
#            return [prepare_docstring(docstring, ignore)]
#        elif docstring:
#            return [prepare_docstring(force_decode(docstring, encoding),
#                                      ignore)]
#        return []
#
#    def process_doc(self, docstrings):
#        """Let the user process the docstrings before adding them."""
#        for docstringlines in docstrings:
#            if self.env.app:
#                # let extensions preprocess docstrings
#                self.env.app.emit('autodoc-process-docstring',
#                                  self.objtype, self.fullname, self.object,
#                                  self.options, docstringlines)
#            for line in docstringlines:
#                yield line
#
#    def add_content(self, more_content, no_docstring=False):
#        """Add content from docstrings, attribute documentation and user."""
#        # set sourcename and add content from attribute documentation
#        if self.analyzer:
#            # prevent encoding errors when the file name is non-ASCII
#            if not isinstance(self.analyzer.srcname, unicode):
#                filename = unicode(self.analyzer.srcname,
#                                   sys.getfilesystemencoding(), 'replace')
#            else:
#                filename = self.analyzer.srcname
#            sourcename = u'%s:docstring of %s' % (filename, self.fullname)
#
#            attr_docs = self.analyzer.find_attr_docs()
#            if self.objpath:
#                key = ('.'.join(self.objpath[:-1]), self.objpath[-1])
#                if key in attr_docs:
#                    no_docstring = True
#                    docstrings = [attr_docs[key]]
#                    for i, line in enumerate(self.process_doc(docstrings)):
#                        self.add_line(line, sourcename, i)
#        else:
#            sourcename = u'docstring of %s' % self.fullname
#
#        # add content from docstrings
#        if not no_docstring:
#            encoding = self.analyzer and self.analyzer.encoding
#            docstrings = self.get_doc(encoding)
#            if not docstrings:
#                # append at least a dummy docstring, so that the event
#                # autodoc-process-docstring is fired and can add some
#                # content if desired
#                docstrings.append([])
#            for i, line in enumerate(self.process_doc(docstrings)):
#                self.add_line(line, sourcename, i)
#
#        # add additional content (e.g. from document), if present
#        if more_content:
#            for line, src in zip(more_content.data, more_content.items):
#                self.add_line(line, src[0], src[1])
#
#    def get_object_members(self, want_all):
#        """Return `(members_check_module, members)` where `members` is a
#        list of `(membername, member)` pairs of the members of *self.object*.
#
#        If *want_all* is True, return all members.  Else, only return those
#        members given by *self.options.members* (which may also be none).
#        """
#        if not want_all:
#            if not self.options.members:
#                return False, []
#            # specific members given
#            ret = []
#            for mname in self.options.members:
#                try:
#                    ret.append((mname, self.get_attr(self.object, mname)))
#                except AttributeError:
#                    self.directive.warn('missing attribute %s in object %s'
#                                        % (mname, self.fullname))
#            return False, ret
#
#        if self.options.inherited_members:
#            # safe_getmembers() uses dir() which pulls in members from all
#            # base classes
#            members = safe_getmembers(self.object)
#        else:
#            # __dict__ contains only the members directly defined in
#            # the class (but get them via getattr anyway, to e.g. get
#            # unbound method objects instead of function objects);
#            # using keys() because apparently there are objects for which
#            # __dict__ changes while getting attributes
#            obj_dict = self.get_attr(self.object, '__dict__')
#            members = [(mname, self.get_attr(self.object, mname, None))
#                       for mname in obj_dict.keys()]
#        membernames = set(m[0] for m in members)
#        # add instance attributes from the analyzer
#        if self.analyzer:
#            attr_docs = self.analyzer.find_attr_docs()
#            namespace = '.'.join(self.objpath)
#            for item in attr_docs.iteritems():
#                if item[0][0] == namespace:
#                    if item[0][1] not in membernames:
#                        members.append((item[0][1], INSTANCEATTR))
#        return False, sorted(members)
#
#    def filter_members(self, members, want_all):
#        """Filter the given member list.
#
#        Members are skipped if
#
#        - they are private (except if given explicitly or the private-members
#          option is set)
#        - they are special methods (except if given explicitly or the
#          special-members option is set)
#        - they are undocumented (except if the undoc-members option is set)
#
#        The user can override the skipping decision by connecting to the
#        ``autodoc-skip-member`` event.
#        """
#        ret = []
#
#        # search for members in source code too
#        namespace = '.'.join(self.objpath)  # will be empty for modules
#
#        if self.analyzer:
#            attr_docs = self.analyzer.find_attr_docs()
#        else:
#            attr_docs = {}
#
#        # process members and determine which to skip
#        for (membername, member) in members:
#            # if isattr is True, the member is documented as an attribute
#            isattr = False
#
#            doc = self.get_attr(member, '__doc__', None)
#            # if the member __doc__ is the same as self's __doc__, it's just
#            # inherited and therefore not the member's doc
#            cls = self.get_attr(member, '__class__', None)
#            if cls:
#                cls_doc = self.get_attr(cls, '__doc__', None)
#                if cls_doc == doc:
#                    doc = None
#            has_doc = bool(doc)
#
#            keep = False
#            if want_all and membername.startswith('__') and \
#                   membername.endswith('__') and len(membername) > 4:
#                # special __methods__
#                if self.options.special_members and membername != '__doc__':
#                    keep = has_doc or self.options.undoc_members
#            elif want_all and membername.startswith('_'):
#                # ignore members whose name starts with _ by default
#                keep = self.options.private_members and \
#                       (has_doc or self.options.undoc_members)
#            elif (namespace, membername) in attr_docs:
#                # keep documented attributes
#                keep = True
#                isattr = True
#            else:
#                # ignore undocumented members if :undoc-members: is not given
#                keep = has_doc or self.options.undoc_members
#
#            # give the user a chance to decide whether this member
#            # should be skipped
#            if self.env.app:
#                # let extensions preprocess docstrings
#                skip_user = self.env.app.emit_firstresult(
#                    'autodoc-skip-member', self.objtype, membername, member,
#                    not keep, self.options)
#                if skip_user is not None:
#                    keep = not skip_user
#
#            if keep:
#                ret.append((membername, member, isattr))
#
#        return ret
#
#    def document_members(self, all_members=False):
#        """Generate reST for member documentation.
#
#        If *all_members* is True, do all members, else those given by
#        *self.options.members*.
#        """
#        # set current namespace for finding members
#        self.env.temp_data['autodoc:module'] = self.modname
#        if self.objpath:
#            self.env.temp_data['autodoc:class'] = self.objpath[0]
#
#        want_all = all_members or self.options.inherited_members or \
#                   self.options.members is ALL
#        # find out which members are documentable
#        members_check_module, members = self.get_object_members(want_all)
#
#        # remove members given by exclude-members
#        if self.options.exclude_members:
#            members = [(membername, member) for (membername, member) in members
#                       if membername not in self.options.exclude_members]
#
#        # document non-skipped members
#        memberdocumenters = []
#        for (mname, member, isattr) in self.filter_members(members, want_all):
#            classes = [cls for cls in AutoDirective._registry.itervalues()
#                       if cls.can_document_member(member, mname, isattr, self)]
#            if not classes:
#                # don't know how to document this member
#                continue
#            # prefer the documenter with the highest priority
#            classes.sort(key=lambda cls: cls.priority)
#            # give explicitly separated module name, so that members
#            # of inner classes can be documented
#            full_mname = self.modname + '::' + \
#                              '.'.join(self.objpath + [mname])
#            documenter = classes[-1](self.directive, full_mname, self.indent)
#            memberdocumenters.append((documenter, isattr))
#        member_order = self.options.member_order or \
#                       self.env.config.autodoc_member_order
#        if member_order == 'groupwise':
#            # sort by group; relies on stable sort to keep items in the
#            # same group sorted alphabetically
#            memberdocumenters.sort(key=lambda e: e[0].member_order)
#        elif member_order == 'bysource' and self.analyzer:
#            # sort by source order, by virtue of the module analyzer
#            tagorder = self.analyzer.tagorder
#            def keyfunc(entry):
#                fullname = entry[0].name.split('::')[1]
#                return tagorder.get(fullname, len(tagorder))
#            memberdocumenters.sort(key=keyfunc)
#
#        for documenter, isattr in memberdocumenters:
#            documenter.generate(
#                all_members=True, real_modname=self.real_modname,
#                check_module=members_check_module and not isattr)
#
#        # reset current objects
#        self.env.temp_data['autodoc:module'] = None
#        self.env.temp_data['autodoc:class'] = None
#
#    def generate(self, more_content=None, real_modname=None,
#                 check_module=False, all_members=False):
#        """Generate reST for the object given by *self.name*, and possibly for
#        its members.
#
#        If *more_content* is given, include that content. If *real_modname* is
#        given, use that module name to find attribute docs. If *check_module* is
#        True, only generate if the object is defined in the module name it is
#        imported from. If *all_members* is True, document all members.
#        """
#        if not self.parse_name():
#            # need a module to import
#            self.directive.warn(
#                'don\'t know which module to import for autodocumenting '
#                '%r (try placing a "module" or "currentmodule" directive '
#                'in the document, or giving an explicit module name)'
#                % self.name)
#            return
#
#        # now, import the module and get object to document
#        if not self.import_object():
#            return
#
#        # If there is no real module defined, figure out which to use.
#        # The real module is used in the module analyzer to look up the module
#        # where the attribute documentation would actually be found in.
#        # This is used for situations where you have a module that collects the
#        # functions and classes of internal submodules.
#        self.real_modname = real_modname or self.get_real_modname()
#
#        # try to also get a source code analyzer for attribute docs
#        try:
#            self.analyzer = ModuleAnalyzer.for_module(self.real_modname)
#            # parse right now, to get PycodeErrors on parsing (results will
#            # be cached anyway)
#            self.analyzer.find_attr_docs()
#        except PycodeError:
#            # no source file -- e.g. for builtin and C modules
#            self.analyzer = None
#            # at least add the module.__file__ as a dependency
#            if hasattr(self.module, '__file__') and self.module.__file__:
#                self.directive.filename_set.add(self.module.__file__)
#        else:
#            self.directive.filename_set.add(self.analyzer.srcname)
#
#        # check __module__ of object (for members not given explicitly)
#        if check_module:
#            if not self.check_module():
#                return
#
#        # make sure that the result starts with an empty line.  This is
#        # necessary for some situations where another directive preprocesses
#        # reST and no starting newline is present
#        self.add_line(u'', '<autodoc>')
#
#        # format the object's signature, if any
#        sig = self.format_signature()
#
#        # generate the directive header and options, if applicable
#        self.add_directive_header(sig)
#        self.add_line(u'', '<autodoc>')
#
#        # e.g. the module directive doesn't have content
#        self.indent += self.content_indent
#
#        # add all content (from docstrings, attribute docs etc.)
#        self.add_content(more_content)
#
#        # document members, if possible
#        self.document_members(all_members)


class ModuleDocumenter(Documenter):
    """
    Specialized Documenter subclass for modules.
    """
    objtype = 'module'
    content_indent = u''
    titles_allowed = True

    option_spec = {
        'members': members_option, 'undoc-members': bool_option,
        'noindex': bool_option, 'inherited-members': bool_option,
        'show-inheritance': bool_option, 'synopsis': identity,
        'platform': identity, 'deprecated': bool_option,
        'member-order': identity, 'exclude-members': members_set_option,
        'private-members': bool_option, 'special-members': bool_option,
    }

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        # don't document submodules automatically
        return False

    def resolve_name(self, modname, parents, path, base):
        if modname is not None:
            self.directive.warn('"::" in automodule name doesn\'t make sense')
        return (path or '') + base, []

    def parse_name(self):
        ret = Documenter.parse_name(self)
        if self.args or self.retann:
            self.directive.warn('signature arguments or return annotation '
                                'given for automodule %s' % self.fullname)
        return ret

    def add_directive_header(self, sig):
        Documenter.add_directive_header(self, sig)

        # add some module-specific options
        if self.options.synopsis:
            self.add_line(
                u'   :synopsis: ' + self.options.synopsis, '<autodoc>')
        if self.options.platform:
            self.add_line(
                u'   :platform: ' + self.options.platform, '<autodoc>')
        if self.options.deprecated:
            self.add_line(u'   :deprecated:', '<autodoc>')

    def get_object_members(self, want_all):
        if want_all:
            if not hasattr(self.object, '__all__'):
                # for implicit module members, check __module__ to avoid
                # documenting imported objects
                return True, safe_getmembers(self.object)
            else:
                memberlist = self.object.__all__
        else:
            memberlist = self.options.members or []
        ret = []
        for mname in memberlist:
            try:
                ret.append((mname, safe_getattr(self.object, mname)))
            except AttributeError:
                self.directive.warn(
                    'missing attribute mentioned in :members: or __all__: '
                    'module %s, attribute %s' % (
                    safe_getattr(self.object, '__name__', '???'), mname))
        return False, ret


class ModuleLevelDocumenter(Documenter):
    """
    Specialized Documenter subclass for objects on module level (functions,
    classes, data/constants).
    """
    def resolve_name(self, modname, parents, path, base):
        if modname is None:
            if path:
                modname = path.rstrip('.')
            else:
                # if documenting a toplevel object without explicit module,
                # it can be contained in another auto directive ...
                modname = self.env.temp_data.get('autodoc:module')
                # ... or in the scope of a module directive
                if not modname:
                    modname = self.env.temp_data.get('py:module')
                # ... else, it stays None, which means invalid
        return modname, parents + [base]


class ClassLevelDocumenter(Documenter):
    """
    Specialized Documenter subclass for objects on class level (methods,
    attributes).
    """
    def resolve_name(self, modname, parents, path, base):
        if modname is None:
            if path:
                mod_cls = path.rstrip('.')
            else:
                mod_cls = None
                # if documenting a class-level object without path,
                # there must be a current class, either from a parent
                # auto directive ...
                mod_cls = self.env.temp_data.get('autodoc:class')
                # ... or from a class directive
                if mod_cls is None:
                    mod_cls = self.env.temp_data.get('py:class')
                # ... if still None, there's no way to know
                if mod_cls is None:
                    return None, []
            modname, cls = rpartition(mod_cls, '.')
            parents = [cls]
            # if the module name is still missing, get it like above
            if not modname:
                modname = self.env.temp_data.get('autodoc:module')
            if not modname:
                modname = self.env.temp_data.get('py:module')
            # ... else, it stays None, which means invalid
        return modname, parents + [base]


class DocstringSignatureMixin(object):
    """
    Mixin for FunctionDocumenter and MethodDocumenter to provide the
    feature of reading the signature from the docstring.
    """

    def _find_signature(self, encoding=None):
        docstrings = Documenter.get_doc(self, encoding, 2)
        if len(docstrings) != 1:
            return
        doclines = docstrings[0]
        setattr(self, '__new_doclines', doclines)
        if not doclines:
            return
        # match first line of docstring against signature RE
        match = py_ext_sig_re.match(doclines[0])
        if not match:
            return
        exmod, path, base, args, retann = match.groups()
        # the base name must match ours
        if not self.objpath or base != self.objpath[-1]:
            return
        # ok, now jump over remaining empty lines and set the remaining
        # lines as the new doclines
        i = 1
        while i < len(doclines) and not doclines[i].strip():
            i += 1
        setattr(self, '__new_doclines', doclines[i:])
        return args, retann

    def get_doc(self, encoding=None, ignore=1):
        lines = getattr(self, '__new_doclines', None)
        if lines is not None:
            return [lines]
        return Documenter.get_doc(self, encoding, ignore)

    def format_signature(self):
        if self.args is None and self.env.config.autodoc_docstring_signature:
            # only act if a signature is not explicitly given already, and if
            # the feature is enabled
            result = self._find_signature()
            if result is not None:
                self.args, self.retann = result
        return Documenter.format_signature(self)


class FunctionDocumenter(DocstringSignatureMixin, ModuleLevelDocumenter):
    """
    Specialized Documenter subclass for functions.
    """
    objtype = 'function'
    member_order = 30

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        return isinstance(member, (FunctionType, BuiltinFunctionType))

    def format_args(self):
        if inspect.isbuiltin(self.object) or \
               inspect.ismethoddescriptor(self.object):
            # cannot introspect arguments of a C function or method
            return None
        try:
            argspec = getargspec(self.object)
        except TypeError:
            # if a class should be documented as function (yay duck
            # typing) we try to use the constructor signature as function
            # signature without the first argument.
            try:
                argspec = getargspec(self.object.__new__)
            except TypeError:
                argspec = getargspec(self.object.__init__)
                if argspec[0]:
                    del argspec[0][0]
        args = inspect.formatargspec(*argspec)
        # escape backslashes for reST
        args = args.replace('\\', '\\\\')
        return args

    def document_members(self, all_members=False):
        pass

    def format_name(self):
        """Format the name of *self.object*.

        This normally should be something that can be parsed by the generated
        directive, but doesn't need to be (Sphinx will display it unparsed
        then).

        Modified for Abjad documentation conventions.
        """
        # normally the name doesn't contain the module (except for module
        # directives of course)
        pieces = self.modname.split('.')
        #return '{}.{}'.format(pieces[-2], pieces[-1])
        modname = pieces[-2:]
        return '.'.join(modname)


class ClassDocumenter(ModuleLevelDocumenter):
    """
    Specialized Documenter subclass for classes.
    """
    objtype = 'class'
    member_order = 20
    option_spec = {
        'members': members_option, 'undoc-members': bool_option,
        'noindex': bool_option, 'inherited-members': bool_option,
        'show-inheritance': bool_option, 'member-order': identity,
        'exclude-members': members_set_option,
        'private-members': bool_option, 'special-members': bool_option,
    }

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        return isinstance(member, class_types)

    def import_object(self):
        ret = ModuleLevelDocumenter.import_object(self)
        # if the class is documented under another name, document it
        # as data/attribute
        if ret:
            if hasattr(self.object, '__name__'):
                self.doc_as_attr = (self.objpath[-1] != self.object.__name__)
            else:
                self.doc_as_attr = True
        return ret

    def format_args(self):
        # for classes, the relevant signature is the __init__ method's
        initmeth = self.get_attr(self.object, '__init__', None)
        # classes without __init__ method, default __init__ or
        # __init__ written in C?
        if initmeth is None or initmeth is object.__init__ or not \
               (inspect.ismethod(initmeth) or inspect.isfunction(initmeth)):
            return None
        try:
            argspec = getargspec(initmeth)
        except TypeError:
            # still not possible: happens e.g. for old-style classes
            # with __init__ in C
            return None
        if argspec[0] and argspec[0][0] in ('cls', 'self'):
            del argspec[0][0]
        return inspect.formatargspec(*argspec)

    def format_name(self):
        """Format the name of *self.object*.

        This normally should be something that can be parsed by the generated
        directive, but doesn't need to be (Sphinx will display it unparsed
        then).

        Modified for Abjad documentation conventions.
        """
        # normally the name doesn't contain the module (except for module
        # directives of course)
        pieces = self.modname.split('.')
        return '{}.{}'.format(pieces[-3], pieces[-1])
        
    def format_signature(self):
        if self.doc_as_attr:
            return ''
        return ModuleLevelDocumenter.format_signature(self)

    def add_directive_header(self, sig):
        if self.doc_as_attr:
            self.directivetype = 'attribute'
        Documenter.add_directive_header(self, sig)

        # add inheritance info, if wanted
        if not self.doc_as_attr and self.options.show_inheritance:
            self.add_line(u'', '<autodoc>')
            if len(self.object.__bases__):
                bases = [b.__module__ == '__builtin__' and
                         u':class:`%s`' % b.__name__ or
                         u':class:`%s.%s`' % (b.__module__, b.__name__)
                         for b in self.object.__bases__]
                self.add_line(_(u'   Bases: %s') % ', '.join(bases),
                              '<autodoc>')

    def get_doc(self, encoding=None, ignore=1):
        content = self.env.config.autoclass_content

        docstrings = []
        attrdocstring = self.get_attr(self.object, '__doc__', None)
        if attrdocstring:
            docstrings.append(attrdocstring)

        # for classes, what the "docstring" is can be controlled via a
        # config value; the default is only the class docstring
        if content in ('both', 'init'):
            initdocstring = self.get_attr(
                self.get_attr(self.object, '__init__', None), '__doc__')
            # for new-style classes, no __init__ means default __init__
            if initdocstring == object.__init__.__doc__:
                initdocstring = None
            if initdocstring:
                if content == 'init':
                    docstrings = [initdocstring]
                else:
                    docstrings.append(initdocstring)
        doc = []
        for docstring in docstrings:
            if not isinstance(docstring, unicode):
                docstring = force_decode(docstring, encoding)
            doc.append(prepare_docstring(docstring))
        return doc

    def add_content(self, more_content, no_docstring=False):
        if self.doc_as_attr:
            classname = safe_getattr(self.object, '__name__', None)
            if classname:
                content = ViewList(
                    [_('alias of :class:`%s`') % classname], source='')
                ModuleLevelDocumenter.add_content(self, content,
                                                  no_docstring=True)
        else:
            ModuleLevelDocumenter.add_content(self, more_content)

    def document_members(self, all_members=False):
        if self.doc_as_attr:
            return
        ModuleLevelDocumenter.document_members(self, all_members)


class ExceptionDocumenter(ClassDocumenter):
    """
    Specialized ClassDocumenter subclass for exceptions.
    """
    objtype = 'exception'
    member_order = 10

    # needs a higher priority than ClassDocumenter
    priority = 10

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        return isinstance(member, class_types) and \
               issubclass(member, base_exception)


class DataDocumenter(ModuleLevelDocumenter):
    """
    Specialized Documenter subclass for data items.
    """
    objtype = 'data'
    member_order = 40
    priority = -10

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        return isinstance(parent, ModuleDocumenter) and isattr

    def add_directive_header(self, sig):
        ModuleLevelDocumenter.add_directive_header(self, sig)
        try:
            objrepr = safe_repr(self.object)
        except ValueError:
            pass
        else:
            self.add_line(u'   :annotation: = ' + objrepr, '<autodoc>')

    def document_members(self, all_members=False):
        pass


class MethodDocumenter(DocstringSignatureMixin, ClassLevelDocumenter):
    """
    Specialized Documenter subclass for methods (normal, static and class).
    """
    objtype = 'method'
    member_order = 50
    priority = 0

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        return inspect.isroutine(member) and \
               not isinstance(parent, ModuleDocumenter)

    if sys.version_info >= (3, 0):
        def import_object(self):
            ret = ClassLevelDocumenter.import_object(self)
            obj_from_parent = self.parent.__dict__.get(self.object_name)
            if isinstance(obj_from_parent, classmethod):
                self.directivetype = 'classmethod'
                self.member_order = self.member_order - 1
            elif isinstance(obj_from_parent, staticmethod):
                self.directivetype = 'staticmethod'
                self.member_order = self.member_order - 1
            else:
                self.directivetype = 'method'
            return ret
    else:
        def import_object(self):
            ret = ClassLevelDocumenter.import_object(self)
            if isinstance(self.object, classmethod) or \
                   (isinstance(self.object, MethodType) and
                    self.object.im_self is not None):
                self.directivetype = 'classmethod'
                # document class and static members before ordinary ones
                self.member_order = self.member_order - 1
            elif isinstance(self.object, FunctionType) or \
                 (isinstance(self.object, BuiltinFunctionType) and
                  hasattr(self.object, '__self__') and
                  self.object.__self__ is not None):
                self.directivetype = 'staticmethod'
                # document class and static members before ordinary ones
                self.member_order = self.member_order - 1
            else:
                self.directivetype = 'method'
            return ret

    def format_args(self):
        if inspect.isbuiltin(self.object) or \
               inspect.ismethoddescriptor(self.object):
            # can never get arguments of a C function or method
            return None
        argspec = getargspec(self.object)
        if argspec[0] and argspec[0][0] in ('cls', 'self'):
            del argspec[0][0]
        return inspect.formatargspec(*argspec)

    def document_members(self, all_members=False):
        pass


class AttributeDocumenter(ClassLevelDocumenter):
    """
    Specialized Documenter subclass for attributes.
    """
    objtype = 'attribute'
    member_order = 60

    # must be higher than the MethodDocumenter, else it will recognize
    # some non-data descriptors as methods
    priority = 10

    method_types = (FunctionType, BuiltinFunctionType, MethodType)

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        isdatadesc = isdescriptor(member) and not \
                     isinstance(member, cls.method_types) and not \
                     type(member).__name__ == "method_descriptor"
        return isdatadesc or (not isinstance(parent, ModuleDocumenter)
                              and not inspect.isroutine(member)
                              and not isinstance(member, class_types))

    def document_members(self, all_members=False):
        pass

    def import_object(self):
        ret = ClassLevelDocumenter.import_object(self)
        if isdescriptor(self.object) and \
               not isinstance(self.object, self.method_types):
            self._datadescriptor = True
        else:
            # if it's not a data descriptor
            self._datadescriptor = False
        return ret

    def get_real_modname(self):
        return self.get_attr(self.parent or self.object, '__module__', None) \
               or self.modname

    def add_directive_header(self, sig):
        ClassLevelDocumenter.add_directive_header(self, sig)
        if not self._datadescriptor:
            try:
                objrepr = safe_repr(self.object)
            except ValueError:
                pass
            else:
                self.add_line(u'   :annotation: = ' + objrepr, '<autodoc>')

    def add_content(self, more_content, no_docstring=False):
        if not self._datadescriptor:
            # if it's not a data descriptor, its docstring is very probably the
            # wrong thing to display
            no_docstring = True
        ClassLevelDocumenter.add_content(self, more_content, no_docstring)


class InstanceAttributeDocumenter(AttributeDocumenter):
    """
    Specialized Documenter subclass for attributes that cannot be imported
    because they are instance attributes (e.g. assigned in __init__).
    """
    objtype = 'instanceattribute'
    directivetype = 'attribute'
    member_order = 60

    # must be higher than AttributeDocumenter
    priority = 11

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        """This documents only INSTANCEATTR members."""
        return isattr and (member is INSTANCEATTR)

    def import_object(self):
        """Never import anything."""
        # disguise as an attribute
        self.objtype = 'attribute'
        self._datadescriptor = False
        return True

    def add_content(self, more_content, no_docstring=False):
        """Never try to get a docstring from the object."""
        AttributeDocumenter.add_content(self, more_content, no_docstring=True)


class AutoDirective(Directive):
    """
    The AutoDirective class is used for all autodoc directives.  It dispatches
    most of the work to one of the Documenters, which it selects through its
    *_registry* dictionary.

    The *_special_attrgetters* attribute is used to customize ``getattr()``
    calls that the Documenters make; its entries are of the form ``type:
    getattr_function``.

    Note: When importing an object, all items along the import chain are
    accessed using the descendant's *_special_attrgetters*, thus this
    dictionary should include all necessary functions for accessing
    attributes of the parents.
    """
    # a registry of objtype -> documenter class
    _registry = {}

    # a registry of type -> getattr function
    _special_attrgetters = {}

    # flags that can be given in autodoc_default_flags
    _default_flags = set([
        'members', 'undoc-members', 'inherited-members', 'show-inheritance',
        'private-members', 'special-members',
    ])

    # standard docutils directive settings
    has_content = True
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    # allow any options to be passed; the options are parsed further
    # by the selected Documenter
    option_spec = DefDict(identity)

    def warn(self, msg):
        self.warnings.append(self.reporter.warning(msg, line=self.lineno))

    def run(self):
        self.filename_set = set()  # a set of dependent filenames
        self.reporter = self.state.document.reporter
        self.env = self.state.document.settings.env
        self.warnings = []
        self.result = ViewList()

        # find out what documenter to call
        objtype = self.name[4:]
        doc_class = self._registry[objtype]
        # add default flags
        for flag in self._default_flags:
            if flag not in doc_class.option_spec:
                continue
            negated = self.options.pop('no-' + flag, 'not given') is None
            if flag in self.env.config.autodoc_default_flags and \
               not negated:
                self.options[flag] = None
        # process the options with the selected documenter's option_spec
        self.genopt = Options(assemble_option_dict(
            self.options.items(), doc_class.option_spec))
        # generate the output
        documenter = doc_class(self, self.arguments[0])
        documenter.generate(more_content=self.content)
        if not self.result:
            return self.warnings

        # record all filenames as dependencies -- this will at least
        # partially make automatic invalidation possible
        for fn in self.filename_set:
            self.state.document.settings.record_dependencies.add(fn)

        # use a custom reporter that correctly assigns lines to source
        # filename/description and lineno
        old_reporter = self.state.memo.reporter
        self.state.memo.reporter = AutodocReporter(self.result,
                                                   self.state.memo.reporter)

        if documenter.titles_allowed:
            node = nodes.section()
            # necessary so that the child nodes get the right source/line set
            node.document = self.state.document
            nested_parse_with_titles(self.state, self.result, node)
        else:
            node = nodes.paragraph()
            node.document = self.state.document
            self.state.nested_parse(self.result, 0, node)
        self.state.memo.reporter = old_reporter
        return self.warnings + node.children


#def add_documenter(cls):
#    """Register a new Documenter."""
#    print 'LOCAL COPY'
#    if not issubclass(cls, Documenter):
#        raise ExtensionError('autodoc documenter %r must be a subclass '
#                             'of Documenter' % cls)
#    # actually, it should be possible to override Documenters
#    #if cls.objtype in AutoDirective._registry:
#    #    raise ExtensionError('autodoc documenter for %r is already '
#    #                         'registered' % cls.objtype)
#    AutoDirective._registry[cls.objtype] = cls


def setup(app):
    app.add_autodocumenter(ModuleDocumenter)
    app.add_autodocumenter(ClassDocumenter)
    app.add_autodocumenter(ExceptionDocumenter)
    app.add_autodocumenter(DataDocumenter)
    app.add_autodocumenter(FunctionDocumenter)
    app.add_autodocumenter(MethodDocumenter)
    app.add_autodocumenter(AttributeDocumenter)
    app.add_autodocumenter(InstanceAttributeDocumenter)

    app.add_config_value('autoclass_content', 'class', True)
    app.add_config_value('autodoc_member_order', 'alphabetic', True)
    app.add_config_value('autodoc_default_flags', [], True)
    app.add_config_value('autodoc_docstring_signature', True, True)
    app.add_event('autodoc-process-docstring')
    app.add_event('autodoc-process-signature')
    app.add_event('autodoc-skip-member')


class testcls:
    """test doc string"""

    def __getattr__(self, x):
        return x

    def __setattr__(self, x, y):
        """Attr setter."""



def list_known_music_functions():
    '''List all music functions recognized by ``LilyPondParser``:

    ::

        >>> for x in lilypondparsertools.list_known_music_functions():
        ...     print x
        ...
        acciaccatura
        appoggiatura
        bar
        breathe
        clef
        grace
        key
        language
        makeClusters
        mark
        relative
        skip
        time
        times
        transpose

    Return list.
    '''
    from abjad.ly import current_module
    from abjad.tools import lilypondparsertools

    music_functions = []
    for name in current_module:
        if isinstance(current_module[name], dict) and 'type' in current_module[name]:
            if current_module[name]['type'] == 'ly:music-function?':
                if hasattr(lilypondparsertools.GuileProxy, name):
                    music_functions.append(name)

    return sorted(music_functions)



def get_improper_descendents_of_component_that_start_with_component(component):
    r'''.. versionadded:: 2.9

    Get improper contents of `component` that start with `component`::

        >>> staff = Staff(r"c' << \new Voice { d' } \new Voice { e' } >> f'")

    ::

        >>> f(staff)
        \new Staff {
            c'4
            <<
                \new Voice {
                    d'4
                }
                \new Voice {
                    e'4
                }
            >>
            f'4
        }

    ::

        >>> componenttools.get_improper_descendents_of_component_that_start_with_component(
        ...     staff[1])
        [<<Voice{1}, Voice{1}>>, Voice{1}, Note("d'4"), Voice{1}, Note("e'4")]

    Return list of `component` together with improper contents that start with component.
    '''
    from abjad.tools import containertools

    # initialize result
    result = []

    # add component
    result.append(component)

    # add content components that start with component
    if isinstance(component, containertools.Container):
        if component.is_parallel:
            for x in component:
                result.extend(get_improper_descendents_of_component_that_start_with_component(x))
        elif component:
            result.extend(get_improper_descendents_of_component_that_start_with_component(component[0]))

    # return result
    return result



from abjad import *


def test_RhythmTreeContainer___call___01():

    rtm = '(1 (1 (2 (1 1 1)) 2))'
    tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
    result = tree((1, 4))

    assert isinstance(result, (list, selectiontools.Selection))
    assert len(result) == 1

    r'''
    \times 4/5 {
        c'16
        \times 2/3 {
            c'16
            c'16
            c'16
        }
        c'8
    }
    '''

    assert result[0].lilypond_format == "\\times 4/5 {\n\tc'16\n\t\\times 2/3 {\n\t\tc'16\n\t\tc'16\n\t\tc'16\n\t}\n\tc'8\n}"


def test_RhythmTreeContainer___call___02():

    rtm = '(1 (1 (2 (1 1 1 1)) 1))'
    tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
    result = tree((1, 4))

    assert isinstance(result, (list, selectiontools.Selection))
    assert len(result) == 6
    assert [x.lilypond_format for x in result] == ["c'16", "c'32", "c'32", "c'32", "c'32", "c'16"]



from abjad.tools import durationtools
from abjad.tools import pitchtools
from abjad.tools import quantizationtools


def test_PitchedQEvent___init___01():

    q_event = quantizationtools.PitchedQEvent(130, [0, 1, 4])

    assert q_event.offset == durationtools.Offset(130)
    assert q_event.pitches == (
        pitchtools.NamedChromaticPitch(0),
        pitchtools.NamedChromaticPitch(1),
        pitchtools.NamedChromaticPitch(4)
        )
    assert q_event.attachments == ()


def test_PitchedQEvent___init___02():

    q_event = quantizationtools.PitchedQEvent(
        durationtools.Offset(133, 5),
        [pitchtools.NamedChromaticPitch('fss')],
        attachments = ['foo', 'bar', 'baz']
        )

    assert q_event.offset == durationtools.Offset(133, 5)
    assert q_event.pitches == (pitchtools.NamedChromaticPitch('fss'),)
    assert q_event.attachments == ('foo', 'bar', 'baz')
        



from abjad.tools import notetools
from abjad.tools import componenttools


def is_neighbor_note(note):
    r'''.. versionadded:: 2.0

    True when `note` is preceeded by a stepwise interval in one direction
    and followed by a stepwise interval in the other direction.
    Otherwise false. ::

        >>> t = Staff("c'8 d'8 e'8 f'8")
        >>> for note in t:
        ...     print '%s\t%s' % (note, tonalitytools.is_neighbor_note(note))
        ...
        c'8     False
        d'8     False
        e'8     False
        f'8     False

    Return boolean.
    '''
    from abjad.tools import tonalitytools

    if not isinstance(note, notetools.Note):
        raise TypeError('must be note: %s' % note)

    try:
        previous_note = componenttools.get_nth_namesake_from_component(note, -1)
        next_note = componenttools.get_nth_namesake_from_component(note, 1)
    except IndexError:
        return False

    notes = [previous_note, note, next_note]

    return tonalitytools.are_stepwise_notes(notes) and not tonalitytools.are_scalar_notes(notes)



from abjad import *


def test_TaleaRhythmMaker_beam_each_cell_01():
    '''Beam each cell with a multipart beam spanner.
    '''

    talea, talea_denominator, prolation_addenda = [1, 1, 1, -1, 2, 2], 32, [3, 4]
    maker = rhythmmakertools.TaleaRhythmMaker(
        talea, talea_denominator, prolation_addenda, beam_each_cell=True)

    divisions = [(2, 16), (5, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    measures = measuretools.make_measures_with_full_measure_spacer_skips(divisions)
    staff = Staff(measures)
    measuretools.replace_contents_of_measures_in_expr(staff, music)
    score = Score([staff])
    score.set.autoBeaming = False

    assert score.lilypond_format == "\\new Score \\with {\n\tautoBeaming = ##f\n} <<\n\t\\new Staff {\n\t\t{\n\t\t\t\\time 2/16\n\t\t\t\\times 4/7 {\n\t\t\t\tc'32 [\n\t\t\t\tc'32\n\t\t\t\tc'32 ]\n\t\t\t\tr32\n\t\t\t\tc'16 [\n\t\t\t\tc'32 ]\n\t\t\t}\n\t\t}\n\t\t{\n\t\t\t\\time 5/16\n\t\t\t\\fraction \\times 5/7 {\n\t\t\t\tc'32 [\n\t\t\t\tc'32\n\t\t\t\tc'32\n\t\t\t\tc'32 ]\n\t\t\t\tr32\n\t\t\t\tc'16 [\n\t\t\t\tc'16\n\t\t\t\tc'32\n\t\t\t\tc'32\n\t\t\t\tc'32 ]\n\t\t\t\tr32\n\t\t\t\tc'32\n\t\t\t}\n\t\t}\n\t}\n>>"



from abjad.tools import contexttools
from abjad.tools import measuretools
from abjad.tools import tuplettools
from abjad.tools.quantizationtools.AttackPointOptimizer import AttackPointOptimizer


class MeasurewiseAttackPointOptimizer(AttackPointOptimizer):
    '''Concrete ``AttackPointOptimizer`` instance which attempts to optimize
    attack points in an expression with regard to the effective time signature
    of that expression.

    Only acts on Measure instances.

    Return ``MeasurewiseAttackPointOptimizer`` instance.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, expr):
        from abjad.tools import quantizationtools

        assert isinstance(expr, measuretools.Measure)
        metrical_hierarchy = timesignaturetools.MetricalHierarchy(expr)
        timesignaturetools.establish_metrical_hierarchy(expr[:], metrical_hierarchy)	



from abjad import *
from abjad.tools.mathtools import NonreducedFraction


def test_durationtools_Duration___sub___01():
    '''Subtracting nonreduced fraction from duration returns nonreduced fraction.
    '''

    result = Duration(1, 2) - NonreducedFraction(2, 8)
    assert isinstance(result, NonreducedFraction)
    assert result.pair == (2, 8)


def test_durationtools_Duration___sub___02():
    '''Subtracting duration from nonreduced fraction returns nonreduced fraction.
    '''

    result = NonreducedFraction(4, 8) - Duration(1, 4)
    assert isinstance(result, NonreducedFraction)
    assert result.pair == (2, 8)



from abjad.tools import durationtools
from abjad.tools import mathtools


def split_component_at_offset(component, offset, 
    fracture_spanners=False, tie_split_notes=True, tie_split_rests=False):
    r'''.. versionadded:: 1.1

    Split `component` at `offset`.

    Example 1. Split `component` at `offset`. Don't fracture spanners::

        >>> staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)

    ::

        >>> beamtools.BeamSpanner(staff[0])
        BeamSpanner(|2/8(2)|)
        >>> beamtools.BeamSpanner(staff[1])
        BeamSpanner(|2/8(2)|)
        >>> spannertools.SlurSpanner(staff.leaves)
        SlurSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> halves = componenttools.split_component_at_offset(
        ... staff.leaves[0], (1, 32), fracture_spanners=False, tie_split_notes=False)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'32 [ (
                c'16.
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Example 2. Split component at offset at fracture crossing spanners::

        >>> staff = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)

    ::

        >>> beamtools.BeamSpanner(staff[0])
        BeamSpanner(|2/8(2)|)
        >>> beamtools.BeamSpanner(staff[1])
        BeamSpanner(|2/8(2)|)
        >>> spannertools.SlurSpanner(staff.leaves)
        SlurSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8 [ (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> halves = componenttools.split_component_at_offset(
        ... staff.leaves[0], (1, 32), fracture_spanners=True, tie_split_notes=False)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'32 ( ) [
                c'16. (
                d'8 ]
            }
            {
                e'8 [
                f'8 ] )
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return pair of left and right part-lists.
    '''
    from abjad.tools import componenttools
    from abjad.tools import containertools
    from abjad.tools import iterationtools
    from abjad.tools import leaftools
    from abjad.tools import measuretools
    from abjad.tools import notetools
    from abjad.tools import resttools
    from abjad.tools import spannertools
    from abjad.tools import tietools
    
    # check input 
    assert isinstance(component, componenttools.Component)
    offset = durationtools.Offset(offset)
    assert 0 <= offset, repr(offset)

    # if zero offset then return empty list and component
    if offset == 0:
        return [], component

    # get split point score offset
    global_split_point = component.start_offset + offset

    # get any duration-crossing descendents
    contents = componenttools.get_improper_descendents_of_component_that_cross_offset(
        component, offset)

    # get any duration-crossing measure descendents
    measures = [x for x in contents if isinstance(x, measuretools.Measure)]

    # if we must split a power-of-two measure at a non-power-of-two split point
    # go ahead and transform the power-of-two measure to non-power-of-two equivalent now;
    # code that crawls and splits later on will be happier
    if len(measures) == 1:
        measure = measures[0]
        split_point_in_measure = global_split_point - measure.start_offset
        if measure.has_non_power_of_two_denominator:
            if not measure.implied_prolation == split_point_in_measure.implied_prolation:
                raise NotImplementedError
        elif not mathtools.is_nonnegative_integer_power_of_two(split_point_in_measure.denominator):
            non_power_of_two_factors = mathtools.remove_powers_of_two(split_point_in_measure.denominator)
            non_power_of_two_factors = mathtools.factors(non_power_of_two_factors)
            non_power_of_two_product = 1
            for non_power_of_two_factor in non_power_of_two_factors:
                non_power_of_two_product *= non_power_of_two_factor
            measuretools.scale_measure_denominator_and_adjust_measure_contents(
                measure, non_power_of_two_product)
            # rederive duration crosses with possibly new measure contents
            contents = componenttools.get_improper_descendents_of_component_that_cross_offset(
                component, offset)
    elif 1 < len(measures):
        raise ContainmentError('measures can not nest.')

    # any duration-crossing leaf will be at end of list
    bottom = contents[-1]

    did_split_leaf = False

    # if split point necessitates leaf split
    if isinstance(bottom, leaftools.Leaf):
        assert isinstance(bottom, leaftools.Leaf)
        did_split_leaf = True
        split_point_in_bottom = global_split_point - bottom.start_offset
        left_list, right_list = leaftools.split_leaf_at_offset(bottom,
            split_point_in_bottom, fracture_spanners=fracture_spanners, 
            tie_split_notes=tie_split_notes, tie_split_rests=tie_split_rests)
        right = right_list[0]
        leaf_right_of_split = right
        leaf_left_of_split = left_list[-1]
        duration_crossing_containers = contents[:-1]
        if not len(duration_crossing_containers):
            return left_list, right_list
    # if split point falls between leaves
    # then find leaf to immediate right of split point
    # in order to start upward crawl through duration-crossing containers
    else:
        duration_crossing_containers = contents[:]
        for leaf in iterationtools.iterate_leaves_in_expr(bottom):
            if leaf.start_offset == global_split_point:
                leaf_right_of_split = leaf
                leaf_left_of_split = leaftools.get_nth_leaf_in_thread_from_leaf(leaf_right_of_split, -1)
                break
        else:
            raise ContainmentError('can not split empty container {!r}.'.format(bottom))

    # find component to right of split that is also immediate child of last duration-crossing container
    for component in componenttools.get_improper_parentage_of_component(leaf_right_of_split):
        if component.parent is duration_crossing_containers[-1]:
            highest_level_component_right_of_split = component
            break
    else:
        raise ValueError('should we be able to get here?')

    # crawl back up through duration-crossing containers and fracture spanners if requested
    if fracture_spanners:
        for x in componenttools.get_improper_parentage_of_component_that_start_with_component(
            leaf_right_of_split):
            spannertools.fracture_spanners_attached_to_component(x, direction=Left)
            if x is component:
                break

    # crawl back up through duration-crossing containers and split each
    prev = highest_level_component_right_of_split
    for duration_crossing_container in reversed(duration_crossing_containers):
        assert isinstance(duration_crossing_container, containertools.Container)
        i = duration_crossing_container.index(prev)
        left, right = containertools.split_container_at_index(
            duration_crossing_container, i, fracture_spanners=fracture_spanners)
        prev = right

    # NOTE: If tie chain here is convenience, then fusing is good.
    #       If tie chain here is user-given, then fusing is less good.
    #       Maybe later model difference between user tie chains and not.
    leaftools.fuse_leaves_in_tie_chain_by_immediate_parent(
        tietools.get_tie_chain(leaf_left_of_split))
    leaftools.fuse_leaves_in_tie_chain_by_immediate_parent(
        tietools.get_tie_chain(leaf_right_of_split))

    # reapply tie here if crawl above killed tie applied to leaves
    if did_split_leaf:
        if  (tie_split_notes and isinstance(leaf_left_of_split, notetools.Note)) or \
            (tie_split_rests and isinstance(leaf_left_of_split, resttools.Rest)):
            if leaf_left_of_split.parentage.root is leaf_right_of_split.parentage.root:
                tietools.apply_tie_spanner_to_leaf_pair(leaf_left_of_split, leaf_right_of_split)

    # return pair of left and right list-wrapped halves of container
    return ([left], [right])



from abjad.tools.abjadbooktools.HTMLOutputFormat import HTMLOutputFormat
from abjad.tools.abjadbooktools.LaTeXOutputFormat import LaTeXOutputFormat
from abjad.tools.abjadbooktools.ReSTOutputFormat import ReSTOutputFormat
from abjad.tools import developerscripttools
import argparse
import os
import sys
import traceback


class AbjadBookScript(developerscripttools.DeveloperScript):

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'book'

    @property
    def long_description(self):
        return '''DESCRIPTION

    abjad-book processes Abjad snippets embedded in HTML, LaTeX, or ReST 
    documents. All Abjad code placed between the <abjad> </abjad> tags in either 
    HTML, LaTeX or ReST type documents is executed and replaced with tags 
    appropriate to the given file type. All output generated by the code snippet
    is captured and inserted in the output file.

    Apart from the special opening and closing Abjad tags, abjad-book also
    has a special line-level suffix tag: `<hide`. All lines ending with the  
    `<hide` tag will be interpreted by Abjad but will not be displayed in the 
    OUTPUT document. 

    The opening <abjad> tag can also be followed by a list of `attribute=value`
    pair. 

    You can make all of an Abjad code block invisible in the output file with
    the following opening tag:

    <abjad>[hide=true]

    This is useful for generating and embedding rendered score images without
    showing any of the Abjad code.

    You can also remove all of the prompts from a code block with the following
    opening tag:

    <abjad>[strip_prompt=true]

    Simply use Abjad's show() function to have Abjad call LilyPond on the Abjad
    snippet and embed the rendered image in the document.

    All Abjad snippets *must* start with no indentation in the document.

EXAMPLES

    1.  Create an HTML, LaTex or ReST document with embedded Abjad code
        between <abjad></abjad> tags. The code *must* be fully flushed 
        to the left, with no tabs or spaces. The content of an HTML file
        with embedded Abjad might look like this:

        This is an <b>HTML</b> document. Here is Abjad code:

        <abjad>
        v = Voice("c'4 d'4 e'4 f'4")
        beamtools.BeamSpanner(v)
        show(v)
        </abjad>

        More ordinary <b>HTML</b> text here.

   2.  Call `abjad-book` on the file just created:

        $ abjad-book file.htm.raw
    '''

    @property
    def output_formats(self):
        return {
            'htm': HTMLOutputFormat,
            'html': HTMLOutputFormat,
            'rst': ReSTOutputFormat,
            'tex': LaTeXOutputFormat,
        }

    @property
    def short_description(self):
        return 'Preprocess HTML, LaTeX or ReST source with Abjad.'

    @property
    def version(self):
        return 2.0

    ### PRIVATE METHODS ###

    def _collect_filenames(self, args):
        filenames = []
        if os.path.isdir(args.path):
            for root, dirs, files in os.walk(args.path):
                for file in files:
                    path = os.path.join(root, file)
                    if self._is_valid_path(path):
                        filenames.append(path)
        else:
            filenames.append(args.path)
        return filenames

    def _is_valid_path(self, path):
        if not os.path.exists(path):
            return False
        if os.path.isfile(path) and \
            path.endswith('.raw') and \
            path.rpartition('.raw')[0].rpartition('.')[-1] in self.output_formats:
            return True
        if os.path.isdir(path):
            return True
        return False

    def _process_filename(self, args, filename):
        from abjad.tools.abjadbooktools.AbjadBookProcessor import AbjadBookProcessor
        input_filename = filename
        print 'Processing {!r}...'.format(os.path.relpath(input_filename))
        try:
            directory = os.path.dirname(input_filename)
            output_filename = input_filename.rpartition('.raw')[0]
            file_extension = output_filename.rpartition('.')[-1]
            image_prefix = os.path.basename(output_filename).rpartition('.')[0]
            with open(input_filename, 'r') as f:
                lines = f.read().split('\n')
            output_format = self.output_formats[file_extension]()
            abjad_book_processor = AbjadBookProcessor(
                directory, lines, output_format,
                skip_rendering=args.skip_rendering, image_prefix=image_prefix,
                verbose=args.verbose)
            processed_lines = abjad_book_processor(verbose=args.verbose)
            print '\tWriting output to {!r}...'.format(os.path.relpath(output_filename))
            with open(output_filename, 'w') as f:
                f.write(processed_lines)
            print '\t...Done!'
        except:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            traceback.print_exc()

    def _validate_path(self, path):
        error = argparse.ArgumentTypeError('{!r} is not a valid path.'.format(path))
        path = os.path.abspath(path)
        if not self._is_valid_path(path):
            raise error
        return path

    ### PUBLIC METHODS ###

    def process_args(self, args):
        for filename in self._collect_filenames(args):
            self._process_filename(args, filename)
        flags = []
        if args.mainline:
            flags.append('-M')
        if args.experimental:
            flags.append('-X')
        if flags:
            developerscripttools.BuildApiScript()(flags)

    def setup_argument_parser(self, parser):

        parser.add_argument('path',
            type=self._validate_path,
            help='the path to process: a filename ending in ".raw" or '
            'an arbitrarily-deep directory tree to be recursed over'
            )

        parser.add_argument('--skip-rendering',
            action='store_true',
            help='skip all image rendering and simply execute the code',
            )

        parser.add_argument('--verbose',
            action='store_true',
            dest='verbose',
            help='run in verbose mode, printing all LilyPond output',
            ) 

        parser.add_argument('-X', '--experimental',
            action='store_true',
            help='rebuild abjad.tools docs after processing',
            )

        parser.add_argument('-M', '--mainline',
            action='store_true',
            help='rebuild mainline docs after processing',
            )



from abjad.tools import componenttools


def all_are_staves(expr):
    '''.. versionadded:: 2.6

    True when `expr` is a sequence of Abjad staves::

        >>> staff = Staff("c'4 d'4 e'4 f'4")

    ::

        >>> stafftools.all_are_staves([staff])
        True

    True when `expr` is an empty sequence::

        >>> stafftools.all_are_staves([])
        True

    Otherwise false::

        >>> stafftools.all_are_staves('foo')
        False

    Return boolean.

    Function wraps ``componenttools.all_are_components()``.
    '''
    from abjad.tools import stafftools

    return componenttools.all_are_components(expr, klasses=(stafftools.Staff,))



import types
from abjad.tools.documentationtools.ReSTDirective import ReSTDirective


class ReSTAutodocDirective(ReSTDirective):
    '''An ReST autodoc directive:

    ::

        >>> autodoc = documentationtools.ReSTAutodocDirective(
        ...     argument='abjad.tools.beamtools.BeamSpanner.BeamSpanner.BeamSpanner',
        ...     directive='autoclass',
        ...     )
        >>> autodoc.options['noindex'] = True
        >>> autodoc
        ReSTAutodocDirective(
            argument='abjad.tools.beamtools.BeamSpanner.BeamSpanner.BeamSpanner',
            directive='autoclass',
            options={
                'noindex': True
                }
            )

    ::

        >>> print autodoc.rest_format
        .. autoclass:: abjad.tools.beamtools.BeamSpanner.BeamSpanner.BeamSpanner
           :noindex:

    Return `ReSTAutodocDirective` instance.
    '''

    ### INITIALIZER ###

    def __init__(self, argument=None, children=None, directive=None, name=None, options=None):
        ReSTDirective.__init__(self, argument=argument, children=children, name=name, options=options)
        self.directive = directive

    ### READ/WRITE PUBLIC PROPERTIES ###

    @apply
    def directive():
        def fget(self):
            return self._directive
        def fset(self, arg):
            assert arg in (
                'autoattribute',
                'autoclass',
                'autodata',
                'autoexception',
                'autofunction',
                'automethod',
                'automodule',
            )
            self._directive = arg
        return property(**locals())



def get_improper_parentage_of_component_that_stop_with_component(component):
    r'''.. versionadded:: 2.9

    Get improper parentage of `component` that stop with `component`::

        >>> staff = Staff(r"c' << \new Voice { d' } \new Voice { e' } >> f'")

    ::

        f(staff)
        \new Staff {
            c'4
            <<
                \new Voice {
                    d'4
                }
                \new Voice {
                    e'4
                }
            >>
            f'4
        }

    ::

        >>> componenttools.get_improper_parentage_of_component_that_stop_with_component(
        ...     staff.leaves[-1])
        [Note("f'4"), Staff{3}]

    Return list of `component` with proper parentage that stop with `component`.
    '''
    from abjad.tools import componenttools
    
    # initialize result
    result = []

    # add component
    result.append(component)

    # add proper paretage that stop with component
    prev = component
    for parent in componenttools.get_proper_parentage_of_component(component):
        if parent.is_parallel:
            if prev.prolated_duration == parent.prolated_duration:
                result.append(parent)
            else:
                break
        elif parent.index(prev) == len(parent) - 1:
            result.append(parent)
        prev = parent

    # return result
    return result



from abjad import *
import py


def test_Measure___setitem___01():

    measure = Measure((3, 4), "c' d' e'")
    measure[:2] = 'r8'

    assert measure.is_underfull
    assert py.test.raises(Exception, 'f(measure)')


def test_Measure___setitem___02():

    measure = Measure((3, 4), "c' d' e'")
    measure.automatically_adjust_time_signature = True
    measure[:2] = 'r8'

    assert not measure.is_underfull
    assert measure.lilypond_format == "{\n\t\\time 3/8\n\tr8\n\te'4\n}"



from abjad.tools import componenttools


def all_are_tuplets(expr):
    '''.. versionadded:: 2.6

    True when `expr` is a sequence of Abjad tuplets::

        >>> tuplet = Tuplet((2, 3), "c'8 d'8 e'8")

    ::

        >>> tuplettools.all_are_tuplets([tuplet])
        True

    True when `expr` is an empty sequence::

        >>> tuplettools.all_are_tuplets([])
        True

    Otherwise false::

        >>> tuplettools.all_are_tuplets('foo')
        False

    Function wraps ``componenttools.all_are_components()``.

    Return boolean.
    '''
    from abjad.tools import tuplettools

    return componenttools.all_are_components(expr, klasses=(tuplettools.Tuplet,))



def get_tab_width():
    r'''.. versionadded:: 2.9

    Get system tab width::

        >>> configurationtools.get_tab_width()
        4

    The value is used by various functions that generate or test code in the system.

    Return nonnegative integer.

    .. versionchanged:: 2.10
        renamed ``configurationtools.get_system_tab_width()`` to
        ``configurationtools.get_tab_width()``.
    '''

    return 4



from abjad.tools import componenttools
from abjad.tools import notetools


def is_passing_tone(note):
    r'''.. versionadded:: 2.0

    True when `note` is both preceeded and followed by scalewise
    sibling notes. Otherwise false. ::

        >>> t = Staff("c'8 d'8 e'8 f'8")
        >>> for note in t:
        ...     print '%s\t%s' % (note, tonalitytools.is_passing_tone(note))
        ...
        c'8     False
        d'8     True
        e'8     True
        f'8     False

    Return boolean.
    '''
    from abjad.tools import tonalitytools

    if not isinstance(note, notetools.Note):
        raise TypeError('must be note: {!r}'.format(note))

    previous_note = componenttools.get_nth_namesake_from_component(note, -1)
    next_note = componenttools.get_nth_namesake_from_component(note, 1)

    if previous_note is None or next_note is None:
        return False

    return tonalitytools.are_scalar_notes(previous_note, note, next_note)



def iterate_sequence_forward_and_backward_overlapping(sequence):
    '''.. versionadded:: 2.0

    Iterate `sequence` first forward and then backward,
    with first and last elements appearing only once::

        >>> list(sequencetools.iterate_sequence_forward_and_backward_overlapping([1, 2, 3, 4, 5]))
        [1, 2, 3, 4, 5, 4, 3, 2]

    Return generator.
    '''

    sequence_copy = []
    for x in sequence:
        yield x
        sequence_copy.append(x)
    for x in reversed(sequence_copy[1:-1]):
        yield x



from abjad.tools import contexttools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools._BrassInstrument import _BrassInstrument
from abjad.tools.instrumenttools._WindInstrument import _WindInstrument


class FrenchHorn(_BrassInstrument, _WindInstrument):
    r'''.. versionadded:: 2.0

    Abjad model of the French horn::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> instrumenttools.FrenchHorn()(staff)
        FrenchHorn()(Staff{4})

    ::

        >>> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Horn }
            \set Staff.shortInstrumentName = \markup { Hn. }
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    The French horn targets staff context by default.
    '''

    def __init__(self, **kwargs):
        _BrassInstrument.__init__(self, **kwargs)
        self._default_instrument_name = 'horn'
        self._default_performer_names.append('hornist')
        self._default_short_instrument_name = 'hn.'
        self._is_primary_instrument = True
        self.sounding_pitch_of_fingered_middle_c = pitchtools.NamedChromaticPitch('f')
        self.primary_clefs = [contexttools.ClefMark('treble'), contexttools.ClefMark('bass')]
        self._copy_primary_clefs_to_all_clefs()
        self._traditional_pitch_range = pitchtools.PitchRange(-25, 17)
